/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.popup;

/** The administration console's imports */
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.core.api.MenuFactory;
import org.objectweb.util.browser.core.api.MenuFactoryConfiguration;
import org.objectweb.util.browser.core.common.DynamicTree;

/**
 * Basic implementation for Menu factories.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 *
 */
public class DefaultMenuFactory
    implements MenuFactory, MenuFactoryConfiguration {

    /** The name of the class (or interface) from which this menu has to be instanciate */
    protected String className_;

    /** Specifies if this menu has to inherit menus from parents at the tree level. */
    protected boolean inheritTreeMenu_;

    /** Specifies if this menu has to inherit menus from parents at the java type inheritance level. */
    protected boolean inheritTypeMenu_;

    /** The menu to create. */
    protected Context items_;

    /** The context containing the tree child configuration */
    protected Context childItems_;

    /** List of context corresponding to the java type inheritance per level */
    protected Context[] inheritTypeMenuFactory_;

    /** The list */
    protected Context[] inheritTreeMenuFactory_;

    /**
     * Add the correxponding item containing in the given Context
     */
    protected void addItems(AdvancedJPopupMenu menu,Context itemList,DynamicTree tree,boolean hisMenu) {
        if (itemList != null) {
            Entry[] entries = itemList.getEntries();
            for (int i = 0; i < entries.length; i++) {
                Entry entry = entries[i];
                ItemProperty ip = (ItemProperty) entry.getValue();
                if (hisMenu || (!hisMenu && ip.isTypeChildVisible()) || (!hisMenu && ip.isTreeChildVisible()))
                    menu.addMenuItem(entry.getName().toString(),ip.getClassName(),tree);
            }
        }
    }

    /**
     * Indicates if a context is empty
     * @return true if the given context is empty
     */
    protected boolean isEmpty(Context context) {
        if (context != null) {
            Entry[] entries = context.getEntries();
            return entries.length == 0;
        }
        return true;
    }

    /**
     * Return a JPopupMenu associated to the given object.
     *
     * @param object The object you want to create the associated menu
     * @param tree The tree on which the popup menu will be displayed
     * @return The associated menu.
     */
    public AdvancedJPopupMenu newMenu(Object object, DynamicTree tree) {

        AdvancedJPopupMenu popupMenu =
            new AdvancedJPopupMenu(tree, inheritTreeMenu_);
        boolean isFirst = true; //To fix correctly the menu separator 

        // Adds its items
        // object.getClass().getName().equals(className_) has boolean to consider the type-child-visible attribute
        addItems(popupMenu,items_,tree,object.getClass().getName().equals(className_));

        // Adds java type inherit items
        if (inheritTypeMenu_ && inheritTypeMenuFactory_ != null) {
            for (int i = 0; i < inheritTypeMenuFactory_.length; i++) {
                if ((isFirst && !isEmpty(items_)) || !isFirst) {
                    popupMenu.addSeparator();
                    isFirst = false;
                }
                addItems(popupMenu, inheritTypeMenuFactory_[i], tree, false);
            }
        }

        return popupMenu;
    }

    /**
     * Return a JPopupMenu associated to the param's object.
     * 
     * @param tree The tree on which the popup menu will be displayed
     * @return The parent menu.
     */
    public AdvancedJPopupMenu newParentMenu(DynamicTree tree) {

        AdvancedJPopupMenu popupMenu = new AdvancedJPopupMenu(tree);
        boolean isFirst = true; //To fix correctly the menu separator 

        // Adds its child items
        addItems(popupMenu, childItems_, tree, false);

        // Adds java type inherit child items
        if (inheritTreeMenuFactory_ != null) {
            for (int i = 0; i < inheritTreeMenuFactory_.length; i++) {
                if ((isFirst && !isEmpty(childItems_)) || !isFirst) {
                    popupMenu.addSeparator();
                    isFirst = false;
                }
                addItems(popupMenu, inheritTreeMenuFactory_[i], tree, false);
            }
        }

        return popupMenu;
    }

    /**
     * Fixes the inheritTreeMenu value.
     * @param inheritTreeMenu true if this menu has to inherit menus from parents at the tree level 
     */
    public void setInheritTreeMenu(boolean inheritTreeMenu) {
        inheritTreeMenu_ = inheritTreeMenu;
    }

    /**
     * Fixes the inheritTypeMenu value.
     * @param inheritTypeMenu true if this menu has to inherit menus from parents at the java type inheritance level 
     */
    public void setInheritTypeMenu(boolean inheritTypeMenu) {
        inheritTypeMenu_ = inheritTypeMenu;
    }

    /**
     * Fixes the list of MenuFactory from which this menu have to inherit.
     * @param inheritMenu The list of MenuFactory
     */
    public void setInheritTypeMenu(Context[] inheritMenu) {
        inheritTypeMenuFactory_ = inheritMenu;
    }

    /**
     * Fixes the list of items for tree child.
     * @param inheritMenu The list of context
     */
    public void setInheritTreeMenu(Context[] inheritMenu) {
        inheritTreeMenuFactory_ = inheritMenu;
    }

    /**
     * Fixes the context containing the popup menu configuration
     * 
     * @param items The context containing the popup menu configuration
     */
    public void setItems(Context items) {
        items_ = items;
    }

    /**
     * Gives the context containing the items for this object
     * @return The context containing this menu configuration
     */
    public Context getItems() {
        return items_;
    }

    /**
     * Fixes the context containing the items for tree child
     * @param childItems The context containing the popup menu configuration for tree child
     */
    public void setChildItems(Context childItems) {
        childItems_ = childItems;
    }

    /**
     * Gives the context containing the items for tree child
     * @return The context containing the tree child configuration
     */
    public Context getChildItems() {
        return childItems_;
    }

    /**
     * Fixes the name of the class from which this menu has to be instanciate.
     * @param className The name of the class
     */
    public void setClassName(String className) {
        className_ = className;
    }

}
