/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2003 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.xmlparser;

/** To use the generated class */
import org.objectweb.util.browser.browserConfig.BrowserUnmarshaller;
import org.objectweb.util.browser.browserConfig.Browser;
import org.objectweb.util.browser.browserConfig.Code;
import org.objectweb.util.browser.browserConfig.DropAction;
import org.objectweb.util.browser.browserConfig.Include;
import org.objectweb.util.browser.browserConfig.Node;
import org.objectweb.util.browser.browserConfig.Icon;
import org.objectweb.util.browser.browserConfig.TypeSystem;
import org.objectweb.util.browser.browserConfig.Wrapper;
import org.objectweb.util.browser.browserConfig.Panel;
import org.objectweb.util.browser.browserConfig.Menu;
import org.objectweb.util.browser.browserConfig.Item;
import org.objectweb.util.browser.browserConfig.Jar;

/** The console's imports */
import org.objectweb.util.browser.core.naming.DefaultEntry;
import org.objectweb.util.browser.core.naming.DefaultName;
import org.objectweb.util.browser.core.naming.DefaultContextContainer;
import org.objectweb.util.browser.core.naming.DefaultContextFactory;
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.IconProvider;
import org.objectweb.util.browser.core.api.BrowserProperty;
import org.objectweb.util.browser.core.api.ContextContainer;
import org.objectweb.util.browser.core.api.ContextProperty;
import org.objectweb.util.browser.core.api.Decoder;
import org.objectweb.util.browser.core.api.DropActionFactory;
import org.objectweb.util.browser.core.api.FileIconProvider;
import org.objectweb.util.browser.core.api.MenuFactoryConfiguration;
import org.objectweb.util.browser.core.api.PanelFactory;
import org.objectweb.util.browser.core.api.SpecificContextFactory;
import org.objectweb.util.browser.core.api.TypeSystemHandler;
import org.objectweb.util.browser.core.common.ClassResolver;
import org.objectweb.util.browser.core.common.DefaultTypeSystemHandler;
import org.objectweb.util.browser.core.common.JavaTypeSystem;
import org.objectweb.util.browser.core.dnd.DefaultDropActionFactory;
import org.objectweb.util.browser.core.icon.DefaultFileIconProvider;
import org.objectweb.util.browser.core.panel.DefaultPanelFactory;
import org.objectweb.util.browser.core.popup.DefaultMenuFactory;
import org.objectweb.util.browser.core.popup.ItemProperty;

/** The java API's imports */
import java.io.File;
import java.io.InputStream;
import java.io.FileInputStream;
import java.util.List;
import java.util.Iterator;
import java.util.Vector;
import java.net.URL;

/**
 * Parses a Java object which represents an XML Configuration. 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class BrowserXMLParser 
    implements XMLParser, BrowserProperty {
	
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The context which contains the XML properties */
    protected ContextContainer context_;

    /** The context property to use */
    protected ContextProperty contextProperty_;

    /** The decoder to use */
    protected Decoder decoder_;

    /** The TypeSystemHandler */
    protected TypeSystemHandler typeSystemHandler_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor
     */
    public BrowserXMLParser() {
        context_ = new DefaultContextContainer();
        typeSystemHandler_ = new DefaultTypeSystemHandler();
    }   

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Add an Include element into the context
     *
     * @param include The Include element to interpret
     */
    protected void addInclude(Include include) {
        InputStream inputStream = null;
        // Tries to load the ressource from the ClassLoader
        inputStream = ClassResolver.getResourceAsStream(include.getUrl());
        if(inputStream==null){
            System.out.println(include.getUrl() + ": resource not found in the ClassLoader");        
            try {
                URL url = new URL(include.getUrl());
                buildBrowser(url.openStream());
            } catch (java.net.MalformedURLException e) {
                if (include.getUrl() != null && !include.getUrl().equals(""))
                    System.out.println(include.getUrl() + ": Malformed URL !");
                else
                    System.out.println("null URL exists in the XML file !");
            } catch (java.io.FileNotFoundException e) {
                System.out.println(include.getUrl() + " : File not found !");
            } catch (java.io.IOException e) {
                System.out.println(e.getMessage());
            }
        } else {
            buildBrowser(inputStream);
        }
    }

    /** 
     * Provide the class corresponding to the code element
     * @param code The code element
     * @return The corresponding class 
     */
    protected Class getClass(Code code) {
        // only Java is supported for the moment !!!
        if(code!=null){
            String className = code.getValue();
            try{
                return ClassResolver.resolve(className);
            }catch(ClassNotFoundException e){
                System.out.println("[BrowserXMLParser.getClass()] " + e + ": ClassNotFoundException");
                return null;   
            }
        }
        return null;
    }

    /**
     * Add a TypeSystem element into the TypeSystemHandler
     * 
     * @param typeSystem The TypeSystem to add
     */
    protected void addTypeSystem(TypeSystem typeSystem) {
        if(typeSystem!=null){
            Class c = getClass(typeSystem.getCode());
            if(org.objectweb.util.browser.core.api.TypeSystem.class.isAssignableFrom(c)){
				try {
					typeSystemHandler_.addTypeSystem((org.objectweb.util.browser.core.api.TypeSystem)c.newInstance());
				} catch (InstantiationException e) {
                    System.out.println("[BrowserXMLParser.addTypeSystem()]");
					e.printStackTrace();
				} catch (IllegalAccessException e) {
                    System.out.println("[BrowserXMLParser.addTypeSystem()]");
					e.printStackTrace();
				}
            }
        }
    }

    /**
     * Creates an Entry.
     *
     * @param value The value of the entry to create.
     * @param id The id of the name of the entry to create.
     *
     * @return A new Entry.
     */
    protected Entry createEntry(String id, Object value) {
        return new DefaultEntry(value, new DefaultName(id), null);
    }

    /**
     * Loads the jar into the Browser CLassLoader
     * @param jarList A list of jar to load
     */
    protected void loadJar(List jarList){
        List urlList = new Vector();
        for (Iterator i = jarList.iterator(); i.hasNext();) {
            Object object = i.next();
            if(object!=null && Jar.class.isAssignableFrom(object.getClass())){
                Jar jar = (Jar)object; 
                String jarFile = jar.getUrl();
                try {
                    urlList.add(new URL(jarFile));
                } catch (java.net.MalformedURLException e) {
                    System.out.println(jarFile + " : Malformed URL !");
                }
            }
        }
        if (!urlList.isEmpty()) {
            ClassResolver.addContext((URL[]) urlList.toArray(new URL[0]));
        }
    }

    /**
     * Looks for the node with the given id
     * @param id The target of the node
     * @return The associated node (or null)
     */
    protected Node getNode(String id) {
        return (Node) context_.getLocalValue(id);
    }

    /**
     * Merges two similar nodes if a node exists in the property context.Else adds the node into the property config
     * If an element of the node is already define in the property context's node, this one is replaced by this which is contained in the given node.
     * @param id Id of the node to merge
     * @param nodeToMerge The node to merge with the one contains in the property context
     */
    protected void mergeConfig(String id, Node nodeToMerge){
        Node node = getNode(id);
        if(node!=null){
            // Merge the wrapper config
            Wrapper wrapper = nodeToMerge.getWrapper();
            if (wrapper != null) 
                node.setWrapper(wrapper);
            // Merge the icon config
            Icon icon = nodeToMerge.getIcon();
            if (icon != null)
                node.setIcon(icon);
            // Merge the panel config
            Panel panel = nodeToMerge.getPanel();
            if (panel != null)
                node.setPanel(panel);
            // Merge the menu config
            Menu menuToMerge = nodeToMerge.getMenu();
            if (menuToMerge != null) {
                Menu menu = node.getMenu();
                if(menu==null)
                    node.setMenu(menuToMerge);
                else{
                    if (menuToMerge.getInheritTreeMenu().equals("false")) 
                        menu.setInheritTreeMenu("false");
                    if (menuToMerge.getInheritTypeMenu().equals("false"));
                        menu.setInheritTypeMenu("false");
                    List itemsToMerge = menuToMerge.getItemList();    
                    if (itemsToMerge!=null && itemsToMerge.size()>0){
                        Iterator it = itemsToMerge.iterator();
                        while (it.hasNext()) {
                            Item item = (Item) it.next();
                            menu.addItem(item);
                        }
                    }
                }
            }
        } else {
            context_.addEntry(id, nodeToMerge);        
        }        
    }

    /** 
     * Parses the XML file and builds the appropriate context
     */
    protected void buildContext(Browser browser) {
        if (browser != null) {
            // Loads jar elements
            loadJar(browser.getJarList());
            // Loads type-system elements
            List typeSystemList = browser.getTypeSystemList();
            for (Iterator i = typeSystemList.iterator(); i.hasNext();) {
                TypeSystem typeSystem = (TypeSystem) i.next();
                addTypeSystem(typeSystem);
            }
            typeSystemHandler_.addTypeSystem(new JavaTypeSystem());
            // Loads include elements
            List includeList = browser.getIncludeList();
            for (Iterator i = includeList.iterator(); i.hasNext();) {
                Include include = (Include) i.next();
                addInclude(include);
            }
            // Loads node elements
            List nodeList = browser.getNodeList();
            for (Iterator i = nodeList.iterator(); i.hasNext();) {
                Node node = (Node) i.next();
                loadJar(node.getJarList());
                String typeSystem = node.getTypeSystem();
                String typeName = node.getTypeName();
                if(typeSystem != null && !typeSystem.equals("") && typeName != null && !typeName.equals("")){
                    mergeConfig(node.getTypeSystem() + "." + node.getTypeName(),node);
                }
            }
        }
    }

    /**
     * Builds the Browser object and builds the context if the created Browser is not null.
     * @param stream A stream opened on the file containing the data defining the Browser
     */
    protected void buildBrowser(InputStream stream) {
        try {
            // We can't validate with DTD at this time (no way to fix the DTD file)
            Browser browser = BrowserUnmarshaller.unmarshal(stream);
            if (browser != null)
                buildContext(browser);
        } catch (java.io.IOException e) {
            System.out.println(e.getMessage());
        }
    }

    /**
     * Builds the Browser object and builds the context if the created Browser is not null.
     * @param file The file containing the data defining the Browser
     */
    protected void buildBrowser(File file) {
        try {
            buildBrowser(new FileInputStream(file));
        } catch (java.io.FileNotFoundException e) {
            System.out.println("Error : " + file + " : File not found !");
        }
    }

    // ==================================================================
    //
    // Public methods of XMLDecoder interface.
    //
    // ==================================================================

    /**
     * Fixes the file which contains the properties.
     * @param path The path containing the XML file(s) containing the configuration 
     * we want to abtain under a Context view (It can be a single file or a directory)
     */
    public void setPropertyFile(String path) {
        InputStream inputStream = null;
        if (path != null) {
            // Tries to load the resource from the ClassLoader
            inputStream = ClassResolver.getResourceAsStream(path);
            if(inputStream==null){            
                // Tries to load the resource from the file system
                try {
                    File file = new File(path);
                    if (file.isDirectory()) {
                        File[] files = file.listFiles(new XMLFileFilter());
                        for (int i = 0; i < files.length; i++) {
                            buildBrowser(files[i]);
                        }
                    } else {
                        buildBrowser(file);
                    }           
                } catch(Exception e1) {
                    System.out.println("[BrowserXMLParser.setPropertyFile] " + path + ": Resource not found on the file system ! ");
                }
            }
            if(inputStream!=null) {
                buildBrowser(inputStream);   
            }
        } else
            System.out.println("The path param is null ! ");
    }

    /**
     * Fixes an array of files (or directories) which contains the properties
     * @param files The path containing the XML file(s) containing the configuration 
     * we want to abtain under a Context view (Each value of the array can be a single file or a directory)
     */
    public void setPropertyFile(String[] files) {
        for (int i = 0; i < files.length; i++)
            setPropertyFile(files[i]);
    }

    // ==================================================================
    //
    // Public methods of BrowserProperty interface.
    //
    // ==================================================================

    /**
     * Fixes the context property to use to constructs the wrappers
     * @param properties The context property to use
     */
    public void setContextProperty(ContextProperty properties) {
        contextProperty_ = properties;
        if (contextProperty_ != null)
            decoder_ = contextProperty_.getDecoder();
    }

    /**
     * Provides the TypeSystem to use in order to find appropriate key.
     * @return The TypeSystem to use.
     */
    public org.objectweb.util.browser.core.api.TypeSystem getTypeSystem(){
        return (org.objectweb.util.browser.core.api.TypeSystem)typeSystemHandler_;
    }

    /**
     * Provides the entry to add into the icon property context or null.
     * @param id The research key
     * @return The entry to add or null
     */
    public Entry getIcon(String id) {
        Node node = getNode(id);
        if (node != null) {
            Icon icon = node.getIcon();
            if (icon != null) {
                if(icon.getIconFile()!=null) {
                    FileIconProvider iconFactory = new DefaultFileIconProvider();
                    iconFactory.setUrl(icon.getIconFile().getUrl());
                    return createEntry(id, iconFactory);
                } else if (icon.getCode()!=null) {
                    try {
                        Class factory = getClass(icon.getCode());
                        if (IconProvider.class.isAssignableFrom(factory))
                            return createEntry(id, factory.newInstance());
                        else
                            System.out.println(icon.getCode().getValue() + " is not an IconProvider instance !");
                    } catch (InstantiationException e) {
                        e.printStackTrace();
                    } catch (IllegalAccessException e) {
                        e.printStackTrace();
                    }
                } else {
                    System.out.println(id + ": An element \"icon-file\" or \"code\" is mandatory in an icon element definition !");
                }
            }
        }
        return null;
    }

    /**
     * Provides the entry to add into the panel property context or null.
     * @param id The research key
     * @return The entry to add or null
     */
    public Entry getPanel(String id) {
        Node node = getNode(id);
        if (node != null) {
            Panel panel = node.getPanel();
            if (panel != null) {
                //System.out.println("[getPanel 1] Thread.currentThread().getContextClassLoader() : " + Thread.currentThread().getContextClassLoader());
                //System.out.println("[getPanel 2] forName with Thread.currentThread().getContextClassLoader()");
                //System.out.println("Thread courant : " + Thread.currentThread());
                Class panelClass = getClass(panel.getCode());
                if (org.objectweb.util.browser.api.Panel.class.isAssignableFrom(panelClass)) {
                    PanelFactory panelFactory = new DefaultPanelFactory();
                    panelFactory.setClassName(panelClass);
                    return createEntry(id, panelFactory);
                } else
                    System.out.println(panel.getCode().getValue() + " is not a Panel instance !");
            }
        }
        return null;
    }

    /**
     * Provides the entry to add into the menu property context or null.
     * @param id The research key (a class name)
     * @return The entry to add or null
     */
    public Entry getMenu(String id) {
        Node node = getNode(id);
        if (node != null) {
            Menu menu = node.getMenu();
            if (menu != null) {
                ContextContainer items = new DefaultContextContainer();
                ContextContainer childItems = new DefaultContextContainer();
                List itemList = menu.getItemList();
                int cpt = 0;
                for (Iterator i = itemList.iterator(); i.hasNext();) {
                    Item item = (Item) i.next();
                    ItemProperty ip = new ItemProperty(item.getLabel(),item.getCode().getValue(),item.getTreeChildVisible().equals("false") ? false : true,item.getTypeChildVisible().equals("true") ? true : false);
                    String label = item.getLabel();
                    if (label!=null && label.equals("-separator-")){
                        label = label + cpt;
                        cpt++;
                    }
                    if (item.getTreeChildVisible().equals("false"))
                        items.addEntry(label, ip);
                    else
                        childItems.addEntry(label, ip);
                }
                MenuFactoryConfiguration menuFactory = new DefaultMenuFactory();
                menuFactory.setInheritTreeMenu(menu.getInheritTreeMenu().equals("false") ? false : true);
                menuFactory.setInheritTypeMenu(menu.getInheritTypeMenu().equals("false") ? false : true);
                menuFactory.setItems(items);
                menuFactory.setChildItems(childItems);
                return createEntry(id, menuFactory);
            }
        }
        return null;
    }

    /**
     * Provides the entry to add into the wrapper property context or null.
     * @param id The research key
     * @return The entry to add or null
     */
    public Entry getWrapper(String id) {
        Node node = getNode(id);
        if (node != null) {
            Wrapper wrapper = node.getWrapper();
            if (wrapper != null) {
                Class wrapperClass = getClass(wrapper.getCode());
                if (org.objectweb.util.browser.api.Wrapper.class.isAssignableFrom(wrapperClass) && Context.class.isAssignableFrom(wrapperClass)) {
                    SpecificContextFactory cf = new DefaultContextFactory(decoder_);
                    cf.setClassName(wrapperClass);
                    return createEntry(id, cf);
                } else
                    System.out.println(wrapper.getCode().getValue() + " is not a Wrapper and a Context instance !");
            }
        }
        return null;
    }

    protected class XMLFileFilter implements java.io.FileFilter {
        public boolean accept(File pathname) {
            String extension = pathname.getName().substring(pathname.getName().lastIndexOf('.') + 1);
            return extension.equalsIgnoreCase("xml") && !pathname.isDirectory();

        }
    }

    /**
     * Provides the entry to add into the dropaction property context or null.
     * @param id The research key
     * @return The entry to add or null
     */
    public Entry getDropAction(String id) {
        Node node = getNode(id);
        if (node != null) {
            DropAction dropAction = node.getDropAction();
            if (dropAction != null) {
                Class actionClass = getClass(dropAction.getCode());
                if (org.objectweb.util.browser.api.DropAction.class.isAssignableFrom(actionClass)) {
                    DropActionFactory dropActionFactory = new DefaultDropActionFactory();
                    dropActionFactory.setLabel(dropAction.getLabel());
                    dropActionFactory.setClassName(actionClass);
                    return createEntry(id, dropActionFactory);
                } else
                    System.out.println(dropAction.getCode().getValue() + " is not a DropAction instance !");
            }
        }
        return null;        
    }

}
