/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.idltree;

import org.w3c.dom.Node;
import java.util.*;
import java.io.*;
import java.lang.reflect.*;

import org.omg.CORBA.ORB;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.TCKind;

/**
 * The class IdlEnum represents an CORBA IDL enum value. Instances are created through one
 * of the create() factory methods in IdlNode.
 * An IdlEnum object has no child nodes.
 * The valid enum values are derived from the typecode.
 *
 * @author <a href="mailto:batteram@lucent.com">Harold Batteram</a> <b>Lucent Technologies</b>
 */
public class IdlEnum extends IdlNode implements IdlWritable {      
    protected String[] members;
    protected int index;

    protected IdlEnum() {
        setUserObject(this);
    }
   
    protected IdlEnum(Any any) {
        this();
        setNode(any);
    }
        
    protected IdlEnum(TypeCode tc) {
        this();
        setNode(tc);
    }

    /**
     * Assigns the node value from a string.
     *
     * @param v The string from which to convert the value.
     */
    public void setValue(String v) {
        boolean found = false;
        for (int i = 0; i < members.length; i++) {
            if (members[i].equals(v)) {
                found = true;
                index = i;
                break;
            }
        }
        if (!found) {
            throw new RuntimeException(v + " is not a valid enumeration in " + id);
        }
        value = v;
    }    

    protected void setNode(TypeCode tc) {
        try {
            this.tc = tc;
            if (tc.kind().value() == TCKind._tk_enum) {
                id = tc.id();
                type = "enum " + id;
                value = tc.member_name(0);
                members = getMembers(tc);
                index = 0;
            } else if (tc.kind().value() == TCKind._tk_alias) {
                id = tc.id();
                type = id;
                TypeCode stc = tc.content_type();
                while (stc.kind().value() == TCKind._tk_alias) {
                    stc = stc.content_type();
                }
                if (stc.kind().value() != TCKind._tk_enum) {
                    throw new RuntimeException("Enum type expected");
                }
                value = stc.member_name(0);
                members = getMembers(stc);
                index = 0;
            } else {
                throw new RuntimeException("Enum type expected");
            }                               
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }

    protected void setNode(Any any) {
        try {
            setNode(any.type());
            org.omg.CORBA.portable.InputStream in = any.create_input_stream();
            index = in.read_long();
            value = tc.member_name(index);
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }

    /**
     * Writes the value to a CORBA outputstream.
     *
     * @param is The outputstream to write to.
     */
    public void write(org.omg.CORBA.portable.OutputStream os) {
        try {
            os.write_long(index);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Reads the value from a CORBA inputstream.
     *
     * @param is The inputstream to read from.
     */
    public void read(org.omg.CORBA.portable.InputStream is) {
        try {
            index = is.read_long();
            value = tc.member_name(index);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Returns the list of valid enumeration values as a String array.
     *
     * @return A String array with all valid enumeration values.
     */    
    public String[] getEnums() {
        return members;
    }

    /**
     * Returns the index number of the current enum value.
     *
     * @return The index number of the current enum value.
     */
    public int getIndex() {
        return index;
    }

    /**
     * Sets the current enum value from the integer index number.
     * Throws a RuntimeException if the given index number is out of range.
     */        
    public void setIndex(int i) {
        if (i < 0 || i >= members.length) {
            throw new RuntimeException("Enum index out of range");
        }
        value = members[i];
        index = i;
    }

    /**
     * Returns the current value as a CORBA Any value.
     *
     * @return The current value as a CORBA Any.
     */
    public Any toAny() {
        try {
            Any any = orb.create_any();
            any.type(tc);
            org.omg.CORBA.portable.OutputStream out = any.create_output_stream();
            out.write_long(index);
            any.read_value(out.create_input_stream(), tc);
            return any;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    private String[] getMembers(TypeCode tc) {
        try {
            String[] members = new String[tc.member_count()];
            for (int i = 0; i < members.length; i++) {
                members[i] = tc.member_name(i);
            }
            return members;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }
    
    // XML section
    
    /**
     * Create an IdlEnum from an Xml representation.
     *
     * XML format example:
     * <pre>
     *   &lt;enum&gt; id="IDL:example/MyEnum" value="blue" &lt;/enum&gt;
     * </pre>
     *
     * @param xml The XML string from which to create an IdlEnum instance.
     */
    public IdlEnum(String xml) {
        this(XmlNode.getNode(xml));
    }
            
    IdlEnum(Node n) {
        this();
        id = XmlNode.getId(n);
        setNode(XmlNode.type(id));
        setValue(XmlNode.getText(n)); 
    }

    /**
     * Write the current value to an IdlWriter object.
     *
     * @param w The IdlWriter object to write the current value to.
     */
    public void write(IdlWriter w) {
        w.write_enum(id, value);
    }
}