/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Wim Hellenthal                                    */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/

package org.coach.tracing.server;

import com.fastcgi.FCGIInterface;
import org.coach.tracing.api.TraceEvent;
import java.io.*;
import java.util.*;

public class ServerMain
{
	private Queue 			queue = new Queue();
    private EventDataBase 	db;

	public ServerMain ()
	{
    	db    = EventDataBase.getEventDB();

        if (System.getProperty("coach.tracing.viewer", "true").equals("true"))
        {
            // Show the internal tracing viewer
            org.coach.tracing.server.viewer.TraceViewerFrame viewer = new org.coach.tracing.server.viewer.TraceViewerFrame();
            viewer.setEventDB(db);
            viewer.show();
        }
	}

    public class Queue
    {
        private LinkedList list = new LinkedList();

        public synchronized void add(TraceEvent[] events)
        {
            list.add(events);

            notifyAll();
        }

        public synchronized TraceEvent[] take()
        {
            if (list.size() == 0)
            {
                try
                {
                    wait();
                }
                catch (Exception e)
                {
                }
            }
            return (TraceEvent[])list.removeFirst();
        }
    }

    public class EventProcessor extends Thread
    {
        public void run()
        {
            while(true)
            {
                db.addEvents(queue.take());
            }
        }
    }

    public class OrbMain extends Thread
    {
    	private org.omg.CORBA.ORB orb;

    	public OrbMain (org.omg.CORBA.ORB orb)
    	{
    		this.orb = orb;
    	}

        public void run()
        {
			orb.run();
        }
    }

    /**
     * This routine initializes the ORB, registers the i_Trace interface at the Name Service, and starts
     * event processing. It also prints information to the standard output stream  This is needed because the FCGI
     * module will reassigns the standard input, output and error stream. If there are no print statements,
     * the launch script (at least for the Windows env.) will keep on testing for a valid TracingServer.output.tmp
     * file.
     *
     * @param args The command line arguments.
     */
	void init (String args[])
	{
		try
		{
			String ns_name = parseArguments (args);

			args = org.objectweb.ccm.CORBA.TheORB.initialize(args);

			org.omg.CORBA.ORB orb = org.objectweb.ccm.CORBA.TheORB.getORB();

			org.omg.CORBA.Object obj = orb.resolve_initial_references( "RootPOA" );

			org.omg.PortableServer.POA rootPOA = org.omg.PortableServer.POAHelper.narrow(obj);

            org.omg.CORBA.Policy [] policies = new org.omg.CORBA.Policy[ 3 ];

            policies[0] = rootPOA.create_id_uniqueness_policy(org.omg.PortableServer.IdUniquenessPolicyValue.MULTIPLE_ID );
            policies[1] = rootPOA.create_request_processing_policy(org.omg.PortableServer.RequestProcessingPolicyValue.USE_DEFAULT_SERVANT);
            policies[2] = rootPOA.create_id_assignment_policy(org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID );

            org.omg.PortableServer.POA childPOA = rootPOA.create_POA("_POA", rootPOA.the_POAManager(), policies );

			i_TraceImpl impl = new i_TraceImpl(queue);

            byte[] servantId = (new String("i_Trace")).getBytes();

			childPOA.activate_object_with_id (servantId, impl);

			obj = orb.resolve_initial_references("NameService");

			org.omg.CosNaming.NamingContext nc = org.omg.CosNaming.NamingContextHelper.narrow(obj);

			obj = childPOA.id_to_reference (servantId);

			org.omg.CosNaming.NameComponent[] ncomp = new org.omg.CosNaming.NameComponent[1];

			ncomp[0] = new org.omg.CosNaming.NameComponent(ns_name, "");

			nc.rebind(ncomp, obj);

			rootPOA.the_POAManager().activate();

			new EventProcessor().start();
			new OrbMain(orb).start();
		}
		catch (Exception _ex)
		{
			_ex.printStackTrace();

			System.exit(-1);
		}
	}

    /**
     * The main method. This method initializes the server and handles the web server FCGI requests.
     *
     * @param args The command line arguments.
     */
    public static void main(String[] args)
    {
        ServerMain application = new ServerMain();

        application.init(args);

		while(new FCGIInterface().FCGIaccept()>= 0)
		{
			application.handleFCGIRequest (System.out, System.getProperty ("QUERY_STRING"));
		}
    }

	void handleFCGIRequest (java.io.OutputStream os, String query)
	{
		// create output print stream

		PrintStream out = new PrintStream (os, true);

		// set content type

		out.println ("Content-type: text/plain\r\n");

		try
		{
			// parse query string

			Hashtable pairs = parseQueryString (query);

			// the event_count request is the only request not returning XML

			if ( !pairs.containsKey("event_count") )
				System.out.println ("<?xml version='1.0' encoding='UTF-8'?>");

			// handle requests

			DbToXml dbc = new DbToXml (db);

			if (pairs.containsKey("entities"))
			{
				StringTokenizer tokenizer = new StringTokenizer((String)pairs.get("keys"));

      			long keys[] = new long[tokenizer.countTokens()];

				int index = 0;

				while (tokenizer.hasMoreTokens())
      			{
					keys[index] = (new Long(tokenizer.nextToken())).longValue();

					index++;
				}

				String result = dbc.getXmlIdentities(keys);

				System.out.println (result);
			}
			else if (pairs.containsKey("event_count"))
			{
				String result = new Long (db.getMessageCount()*2).toString();

				System.out.println (result);
			}
			else if (pairs.containsKey("events") )
			{
				long start 	= Long.parseLong((String)pairs.get ("start"));
				int  len 	= Integer.parseInt((String)pairs.get ("length"));

				StringTokenizer tokenizer = new StringTokenizer((String)pairs.get("operation_filter"));
				Hashtable operation_filter = new Hashtable();

				while (tokenizer.hasMoreTokens())
					operation_filter.put (tokenizer.nextToken(),"");

				tokenizer  = new StringTokenizer((String)pairs.get("entity_filter"));
				Hashtable entity_filter = new Hashtable();

				while (tokenizer.hasMoreTokens())
					entity_filter.put(tokenizer.nextToken(),"");

				tokenizer  = new StringTokenizer((String)pairs.get("toself_filter"));
				Hashtable toself_filter = new Hashtable();

				while (tokenizer.hasMoreTokens())
				{
					StringTokenizer collapsed_node = new StringTokenizer(tokenizer.nextToken(), ":");

					String key 		= new String();
					String value 	= new String();

					if (collapsed_node.hasMoreTokens())
		            	key = collapsed_node.nextToken();
					else;

		          	if (collapsed_node.hasMoreTokens())
		            	value = collapsed_node.nextToken();
		          	else;

					toself_filter.put(key,value);
				}

				String  filter_unmatched = (String)pairs.get ("unmatched_filter");
                long    hide_to 	= Long.parseLong((String)pairs.get ("hide_to"));

				String result = dbc.getXmlEvents(start,len, operation_filter, entity_filter, toself_filter, filter_unmatched, hide_to);

				System.out.println (result);
			}
			else if (pairs.containsKey("parameters") )
			{
				String result = dbc.getXmlParameterValues((new Long((String)pairs.get("key"))).longValue());

				System.out.println (result);
			}

			out.close();
		}
		catch (Exception _ex)
		{
			// print exception message to web server log file

			System.err.println (_ex.getMessage());
		}
	}

    /**
     * Parses two program options in random order. -ns_name <name> is the name used to register at the name service,
     * -fcgi_port <number> is the port the server will listen for web server FCGI request.
     *
     * @param args The command line arguments.
     */
    String parseArguments (String args[])
    {
    	String  name = "COACH_TracingServer";
    	int 	index;

		// if there are no arguments we need to print at least one line.

		System.out.println (".................................");

    	for (index = 0; index < args.length; index++)
		{
			if (args[index].charAt(0) == '-')
			{
				if ( args[index].equals ("-ns_name") )
				{
					if ( (++index == args.length) || (args[index].charAt(0) == '-') )
						System.out.println ("-ns_name option missing argument, using default name '" + name + "'");
					else
					{
						System.out.println ("Registering server using name: " + args[index]);

						name = new String(args[index]);
					}
				}
				else if ( args[index].equals ("-fcgi_port")  )
				{
					if ( (++index == args.length) || (args[index].charAt(0) == '-') )
						System.out.println ("-fcgi_port option missing argument, using default port " + System.getProperty ("FCGI_PORT"));
					else
					{
						System.out.println ("Using FCGI port " + args[index]);

						System.setProperty ("FCGI_PORT", args[index]);
					}
				}
				else
					System.out.println ("Illegal option " + args[index]);
			}
			else
				break;
		}

		return name;
	}

	String urlDecode(String in)
	{
		StringBuffer out = new StringBuffer(in.length());
		int i = 0;
		int j = 0;

		while (i < in.length())
		{
			char ch = in.charAt(i);
			i++;

			if (ch == '+')
				ch = ' ';
			else if (ch == '%')
			{
				ch = (char)Integer.parseInt(in.substring(i,i+2), 16);
				i+=2;
			}
			out.append(ch);
			j++;
		}

		return new String(out);
	}

	Hashtable parseQueryString(String s)
	{
		Hashtable pairs = new Hashtable();

		StringTokenizer pair_tokenizer = new StringTokenizer(s, "&");

      	while (pair_tokenizer.hasMoreTokens())
      	{
          	String pair = urlDecode(pair_tokenizer.nextToken());

			StringTokenizer keyval_tokenizer = new StringTokenizer(pair, "=");

			String key 		= new String();
			String value 	= new String();

			if (keyval_tokenizer.hasMoreTokens())
            	key = keyval_tokenizer.nextToken();
			else;

          	if (keyval_tokenizer.hasMoreTokens())
            	value = keyval_tokenizer.nextToken();
          	else;

			pairs.put(key,value);
		}

		return pairs;
	}
}

