package org.coach.tracing.server.viewer;

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.util.*;
import org.coach.tracing.api.*;
import org.coach.tracing.server.EventDataBase;

public class IdentityNode extends DefaultMutableTreeNode {
    // Facet reference to the remote database.
    private static EventDataBase eventDB;
    // Maintain all identities in a linear map for fast lookup.
    private static HashMap identityMap = new HashMap();
    // display order list
    private static LinkedList identityList = new LinkedList();
    // Array of currently visible identies.
    private static IdentityNode[] visibles;

    private Identity identity;
        
    private boolean isExpanded = true;
    private boolean isCollapsed = false;
    private boolean isHidden = false;

    // The root of the identity hierarchy.
    private static IdentityNode root;

    /**
     * private constructor. The identity tree structure is created through the
     * static get operation.
     */    
    private IdentityNode(Identity id) {
        identity = id;
        setUserObject(this);
        // invalidate the list of visible identities.
        visibles = null;
    }

    /**
     * Sets the event database reference and initializes the identity hierarchy.
     */
    public static void setEventDB(EventDataBase db) {
        eventDB = db;
        identityMap.clear();
        identityList.clear();
        root = new IdentityNode(new Identity(IdentityKind.CCM_NODE, "root", "", (long)-1));
        visibles = null;        
    }
            
    /**
     * Search for an Identity object in the hierarchy tree.
     * If the key is not in the tree the identity is obtained from the database.
     */    
    public static IdentityNode get(Long key) {
        if (key == null || key.longValue() == -1) {
            return root;
        }
        IdentityNode n = (IdentityNode)identityMap.get(key);
        if (n == null) {
            try {
                // The identity is not in the map, read it from the database.
                Identity id = eventDB.getIdentity(key);
                n = new IdentityNode(id);
                
                // Obtain the parent node and add the new identity as its child.
                IdentityNode parent = get(new Long(id.linkKey));
                parent.add(n);
                
                // Update the identity map for fast lookup.
                identityMap.put(key, n);
                identityList.add(n);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return n;
    }       

    public static IdentityNode[] getVisibles() {
        if (visibles == null) {       
            LinkedList s = new LinkedList();
            Iterator it = identityList.iterator();
            while(it.hasNext()) {
                IdentityNode n = (IdentityNode)it.next();
                if (n.isVisible()) {
                    s.add(n);
                }
            }
            visibles = new IdentityNode[s.size()];
            s.toArray(visibles);
        }
        return visibles;
    }
    
    public static void moveLeft(IdentityNode n) {
        int idx = identityList.indexOf(n);
        if (idx > 0) {
            identityList.remove(idx);
            identityList.add(idx - 1, n);
            if (!((IdentityNode)identityList.get(idx)).isVisible()) {
                // The previous node was not visible, so move again.
                // Otherwise the user does not see the move on his screen.
                moveLeft(n);
            }
            visibles = null;
        }
    }

    public static void moveRight(IdentityNode n) {
        int idx = identityList.indexOf(n);
        if (idx >= 0 && idx < identityList.size() - 1) {
            identityList.remove(idx);
            identityList.add(idx + 1, n);
            if (!((IdentityNode)identityList.get(idx)).isVisible()) {
                // The previous node was not visible, so move again.
                // Otherwise the user does not see the move on his screen.
                moveRight(n);
            }
            visibles = null;
        }
    }

    public static void unhideAll() {
        root.setHidden(false);
    }

    /**
     * Return the root of the identity hierarchy.
     * The root is only a holder for the tree and does not represent an identity itself.
     */
    public static IdentityNode getRootNode() {
        return root;
    }
    
    public IdentityKind getKind() {
        return identity.kind;
    }

    public String getName() {
        return identity.name;
    }

    public String getType() {
        return identity.type;
    }

    public String toString() {
        return getShortName() + " " + isVisible();
    }

    public String getShortName() {
        String result = "";

        //trim from end to rightmost dot.
        int index = identity.name.indexOf("@");
        if (index > 0) {
            result = identity.name.substring(0, index);
            index = result.lastIndexOf("/");
            if (index > 0) {
                result = result.substring(index + 1);
            }
            return result;
        }

        //trim from end to rightmost dot.
        index = identity.name.lastIndexOf("/");

        if ((index > 0) && (index < identity.name.length() - 1)) {
            result = identity.name.substring(index + 1);
        } else {
            result = identity.name;
        }
        return result;
    }

    // If a node is hidden, all its children are hidden too.        
    public void setHidden(boolean hide) {
        this.isHidden = hide;
        int cnt = getChildCount();
        for (int i = 0; i < cnt; i++) {
            IdentityNode mo = (IdentityNode)getChildAt(i);
            mo.setHidden(hide);
        } 
        // invalidate the list of visible identities.
        visibles = null;
    }
    
    public void expand() {
        int cnt = getChildCount();
        for (int i = 0; i < cnt; i++) {
            IdentityNode n = (IdentityNode)getChildAt(i);
            n.collapseChildren();
        }
        isExpanded = true;
        isCollapsed = false;
        // invalidate the list of visible identities.
        visibles = null;
    }

    /**
     * When a node is collapsed, its parent becomes visible.
     */
    public void collapse() {
        if (!isRoot()) {
            IdentityNode p = (IdentityNode)getParent();
            p.collapseChildren();             
        }        
        // invalidate the list of visible identities.
        visibles = null;
    }

    /**
     * Collapse the whole subtree.
     */
    private void collapseChildren() {
        if (!isRoot()) {
            Enumeration e = depthFirstEnumeration();
            while(e.hasMoreElements()) {
                IdentityNode n = (IdentityNode)e.nextElement();        
                n.isCollapsed = true;
            }
            isCollapsed = false;
            isExpanded = false;
        }
    }

    public boolean isVisible() {
        if (isRoot() || isHidden || isCollapsed) {
            return false;
        }
        
        if (!isLeaf() && isExpanded) {
            return false;
        }

        return true;
    }
    
    /**
     * Returns the node that is used to map the GUI for this node.
     */
    public IdentityNode getVisible() {
        if (isRoot()) {
            return null;
        }
        
        if (isVisible()) {
            return this;
        } else {
            IdentityNode p = (IdentityNode)getParent();
            return p.getVisible();
        }
    }
       
    public boolean isHidden() {
        if (isRoot()) {
            return false;
        }
        
        if (isHidden) {
            return true;
        }
            
        return ((IdentityNode)getParent()).isHidden();
    }       

    public static String getObjectName(Long key) {
        return get(key).getName();
    }

    public static String getObjectType(Long key) {
        return get(key).getType();
    }
    
    public static String getProcessId(Long key) {
        IdentityNode n = get(key);
        while (n.getKind() != IdentityKind.CCM_PROCESS) {
            n = (IdentityNode)n.getParent();
            if (n.isRoot()) {
                return "";
            }
        }
        return n.getName();
    }

    public static String getNodeName(Long key) {
        IdentityNode n = get(key);
        while (n.getKind() != IdentityKind.CCM_NODE) {
            n = (IdentityNode)n.getParent();
            if (n.isRoot()) {
                return "";
            }
        }
        return n.getName();
    }

    public static String getNodeIp(Long key) {
        IdentityNode n = get(key);
        while (n.getKind() != IdentityKind.CCM_NODE) {
            n = (IdentityNode)n.getParent();
            if (n.isRoot()) {
                return "";
            }
        }
        return n.getType();
    }

   public static String getCmpName(Long key) {
        IdentityNode n = get(key);
        while (n.getKind() != IdentityKind.CCM_COMPONENT) {
            n = (IdentityNode)n.getParent();
            if (n.isRoot()) {
                return "";
            }
        }
        return n.getName();
    }

    public static String getCmpType(Long key) {
        IdentityNode n = get(key);
        while (n.getKind() != IdentityKind.CCM_COMPONENT) {
            n = (IdentityNode)n.getParent();
            if (n.isRoot()) {
                return "";
            }
        }
        return n.getType();
    }

    public static String getCntName(Long key) {
        IdentityNode n = get(key);
        while (n.getKind() != IdentityKind.CCM_CONTAINER) {
            n = (IdentityNode)n.getParent();
            if (n.isRoot()) {
                return "";
            }
        }
        return n.getName();
    }

    public static String getCntType(Long key) {
        IdentityNode n = get(key);
        while (n.getKind() != IdentityKind.CCM_CONTAINER) {
            n = (IdentityNode)n.getParent();
            if (n.isRoot()) {
                return "";
            }
        }
        return n.getType();
    }
}
