/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.deploytool;

/**
 * This is the CCM XML connections-maker operator.
 *
 * @author <a href="mailto:Christophe.Contreras@lifl.fr">Christophe Contreras</a>
 *
 * @version 0.1
 */

public class Connector
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the calling process. */
    private DeploymentProcess _deployer;

    /** Reference to the connections hashtable. */
    private java.util.Hashtable connections;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     *  The default constructor.
     * initializes the table, and wait for "ready" notifications
     *
     * @param deployer the originating deployment process
     */
    public
    Connector (DeploymentProcess deployer)
    {
        // attaching to the deployment process
        this._deployer = deployer;

        _deployer.getConsole().message("Initiating the connector...");

        // initializing the connections hashtable
        connections
            = new java.util.Hashtable();

        // Getting the assembly descriptor
        org.objectweb.ccm.descriptor.componentassembly.beans.ComponentassemblyBean assemblyObject
            = _deployer
                .getAssemblyObject();

        // get the connectinterfaces list
        java.util.List connectInterfacesList
            = assemblyObject
                .getConnections()
                .getConnectinterfaceList();

        for (java.util.Iterator i = connectInterfacesList.iterator() ; i.hasNext() ; )
        {
            org.objectweb.ccm.descriptor.componentassembly.beans.ConnectinterfaceBean connectInterface
                = (org.objectweb.ccm.descriptor.componentassembly.beans.ConnectinterfaceBean) i.next();

            // feed the connections table. false mean no party is ready in the connection
            connections.put(
                connectInterface
                ,new Boolean (false)
                );
        }

        _deployer.getConsole().message("Connector got <connectinterface> list");

        // get the connectevents list
        java.util.List connectEventsList
            = assemblyObject
                .getConnections()
                .getConnecteventList();

        for (java.util.Iterator i = connectEventsList.iterator() ; i.hasNext() ; )
        {
            org.objectweb.ccm.descriptor.componentassembly.beans.ConnecteventBean connectEvent
                = (org.objectweb.ccm.descriptor.componentassembly.beans.ConnecteventBean) i.next();

            // feed the connections table. false mean no party is ready in the connection
            connections.put(
                connectEvent
                , new Boolean(false)
                );
        }
        _deployer.getConsole().message("Connector got <connectevent> list");

        _deployer.getConsole().message("Connector is ready");
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Connect an user to a provider
     *  called when any party is ready
     *  if the other is still missing, set a flag
     *  at second party's notification, does the connection
     *
     * @param connectInterface The Java version of the descriptor
     *
     * @return a boolean indicating success of the connection
     */
    private boolean
    doConnectInterface (
        org.objectweb.ccm.descriptor.componentassembly.beans.ConnectinterfaceBean connectInterface
        )
    {
        if ( ( (Boolean)connections.get(connectInterface) ).booleanValue() && true)
        {
            // get the provider object from the partition
            org.omg.Components.CCMObject provider
                =(org.omg.Components.CCMObject)
                    _deployer
                        .getPartition()
                        .get(
                            connectInterface
                                .getProvidesport()
                                .getComponentinstantiationref()
                                .getIdref()
                            );

            // initialize a provided resource
            org.omg.CORBA.Object providedObject = null;
            
            try {
                // get the provided resource
                providedObject
                    = provider
                        .provide_facet(
                            connectInterface
                                .getProvidesport()
                                .getProvidesidentifier()
                                .getValue()
                            );
            } catch (Exception exception) {
                _deployer.getConsole().error("Wrong <providesidentifier> defintion");
                try {
                    connectInterface
                        .marshal(
                            _deployer
                                .getConsole().getErrorStream()
                            );
                } catch (java.io.IOException io_exception) {
                    _deployer
                        .getConsole().error(
                            "XML description couldn't be displayed during this error reporting"
                            );
                }
                return false;
            }

            // get the user object from the partition
            org.omg.Components.CCMObject user
                = (org.omg.Components.CCMObject)
                    _deployer
                        .getPartition()
                        .get(
                            connectInterface
                                .getUsesport()
                                .getComponentinstantiationref()
                                .getIdref()
                            );
            try {
                // call the connect method
                user
                    .connect(
                        connectInterface
                            .getUsesport()
                            .getUsesidentifier()
                            .getValue()
                        ,providedObject
                        );
            } catch (Exception exception) {
                _deployer
                    .getConsole().error(
                        "Error during a connection, trying to display the erroneous XML"
                        );
                try {
                    connectInterface
                        .marshal(
                            _deployer
                                .getConsole().getErrorStream()
                            );
                } catch (java.io.IOException io_exception) {
                    _deployer
                        .getConsole().error(
                            "XML description couldn't be displayed during this error reporting"
                            );
                }
                return false;
            }
        }else{
            // was the first call for this connection. set up the flag
            connections
                .put(
                    connectInterface
                    , new Boolean(true)
                    );
        }

        // everything was OK during connection
        return true;
    }

    /**
     * Subscribe a component to a publisher
     *  called when any party is ready
     *  if the other is still missing, set a flag
     *  at second party's call, connect
     *
     * @param connectEvent The Java version of the descriptor
     *
     * @return a boolean indicating success of the connection
     */
    private boolean
    doConnectEvent (
        org.objectweb.ccm.descriptor.componentassembly.beans.ConnecteventBean connectEvent
        )
    {
        if ( ( (Boolean)connections.get(connectEvent) ).booleanValue() && true)
        {
            // get the consumer object from the partition
            org.omg.Components.CCMObject consumer
                =(org.omg.Components.CCMObject)
                    _deployer
                        .getPartition()
                        .get(
                            connectEvent
                                .getConsumesport()
                                .getComponentinstantiationref()
                                .getIdref()
                            );

            // initialize a consumed resource
            org.omg.Components.EventConsumerBase consumedObject = null;

            try {
                // get the consumed resource
                consumedObject
                    =consumer.get_consumer(
                        connectEvent
                            .getConsumesport()
                            .getConsumesidentifier()
                            .getValue()
                        );
            } catch (Exception exception) {
                _deployer.getConsole().error("Wrong <consumesidentifier> defintion");
                try {
                    connectEvent
                        .marshal(
                            _deployer
                                .getConsole().getErrorStream()
                            );
                } catch (java.io.IOException io_exception) {
                    _deployer
                        .getConsole().error(
                            "XML description couldn't be displayed during this error reporting"
                            );
                }
                return false;
            }

            // get the subscriber object from the partition
            org.omg.Components.CCMObject subscriber
                = (org.omg.Components.CCMObject)
                    _deployer
                        .getPartition()
                        .get(
                            connectEvent
                                .getPublishesport()
			        .getComponentinstantiationref()
			        .getIdref()
                            );

            try {
                // call the subscribe method
                subscriber
                    .subscribe(
                        connectEvent
                            .getPublishesport()
                            .getPublishesidentifier()
                            .getValue()
                        ,consumedObject
                        );
            } catch (Exception exception) {
                _deployer.getConsole().error("Error during subscription for event");
                try {
                    connectEvent
                        .marshal(
                            _deployer
                                .getConsole().getErrorStream()
                            );
                } catch (java.io.IOException io_exception) {
                    _deployer
                        .getConsole().error(
                            "XML description couldn't be displayed during this error reporting"
                            );
                }
                return false;
            }
        }else{

            // was the first call for this connection. set up the flag
            connections
                .put(
                    connectEvent
                    , new Boolean(true)
                    );
        }

        // everything was OK during connection
        return true;
    }

    /**
     * Search the connections table
     *  for a connection where componentId is involved
     *
     * @param componentId the ID of the involved component
     * @param connectionType the kind of connection wanted
     *
     * @return a list of connections references
     */
    private java.util.List
    searchForConnections (
        String componentId
        , String connectionType)
    {
        java.util.List returnedList
            = new java.util.LinkedList();

        if (connectionType.equals("connectinterface"))
        {
            java.util.List connectInterfacesList
                = _deployer
                    .getAssemblyObject()
                    .getConnections()
                    .getConnectinterfaceList();

            for (java.util.Iterator i = connectInterfacesList.iterator() ; i.hasNext() ; )
            {
                org.objectweb.ccm.descriptor.componentassembly.beans.ConnectinterfaceBean connectInterface
                    = (org.objectweb.ccm.descriptor.componentassembly.beans.ConnectinterfaceBean) i.next();
                if (componentId.equals(connectInterface.getProvidesport().getComponentinstantiationref().getIdref()))
                {
                    returnedList
                        .add(connectInterface);
                }
                if (componentId.equals(connectInterface.getUsesport().getComponentinstantiationref().getIdref()))
                {
                    returnedList
                        .add(connectInterface);
                }
            }
        }

        if (connectionType.equals("connectevent"))
        {
            java.util.List connectEventsList
                = _deployer
                    .getAssemblyObject()
                    .getConnections()
                    .getConnecteventList();

            for (java.util.Iterator i = connectEventsList.iterator() ; i.hasNext() ; )
            {
                org.objectweb.ccm.descriptor.componentassembly.beans.ConnecteventBean connectEvent
                    = (org.objectweb.ccm.descriptor.componentassembly.beans.ConnecteventBean) i.next();

                if (componentId.equals(connectEvent.getConsumesport().getComponentinstantiationref().getIdref()))
                {
                    returnedList
                        .add(connectEvent);
                }
                if (componentId.equals(connectEvent.getPublishesport().getComponentinstantiationref().getIdref()))
                {
                    returnedList
                        .add(connectEvent);
                }
            }
        }
    return returnedList;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Called by the Partitionner at component creation
     *  looks for connections to do
     *  call the methods for each found
     *
     * @param componentId The ID of the component
     *
     * @return a boolean indicating success
     */
    public boolean
    componentReady (String componentId)
    {
        java.util.List concernedConnections = searchForConnections(componentId, "connectinterface");
	
        for (java.util.Iterator i = concernedConnections.iterator() ; i.hasNext() ; )
        {
            org.objectweb.ccm.descriptor.componentassembly.beans.ConnectinterfaceBean connectInterface
                = (org.objectweb.ccm.descriptor.componentassembly.beans.ConnectinterfaceBean) i.next();

            if ( !
                doConnectInterface(connectInterface)
                )
                return false;
        }    

        concernedConnections = searchForConnections(componentId, "connectevent");

        for (java.util.Iterator i = concernedConnections.iterator() ; i.hasNext() ; )
        {
            org.objectweb.ccm.descriptor.componentassembly.beans.ConnecteventBean connectEvent
                = (org.objectweb.ccm.descriptor.componentassembly.beans.ConnecteventBean) i.next();

            if ( !
                doConnectEvent(connectEvent)
                )
                return false;
        }

        // everything was OK during the component connections
        return true;
    }
}
