/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): Philippe Merle.

====================================================================*/

package org.objectweb.ccm.deploytool;

// Package dependencies
import org.objectweb.util.cmdline.api.CommandLine;
import org.objectweb.util.cmdline.lib.ApplicationBase;
import org.objectweb.util.cmdline.lib.DefaultCommandLine;


/**
 * This is the CCM XML deployment application.
 *
 * @author <a href="mailto:Christophe.Contreras@lifl.fr">Christophe Contreras</a>
 *
 * @version 0.1
 */

public class DeploymentApplication
     extends ApplicationBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the deployer. */
    private DeploymentProcess _deployer;
    /*The name of the zip file to deploy*/
    private String zipFile ;
    /*Boolean denoting if the deployment must be distributed or not*/
    private boolean distributedDeployment =false;
    /*In distributed deployment case the assembly factory name to use*/
    private String assemblyFactoryName =null;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     *  The default constructor.
     */
    public
    DeploymentApplication()
    {
        this(
            new DefaultCommandLine(
                 new String[]{"ccm_deploy","ccm_deploy"}
                ,new String[]{"-Fassembly_factory_name","zip_file.zip"}
                ,new String[]{"The AssemblyFactory to use","The zip file to deploy"}
                ,false
                )
            );
    }

    /**
     * The constructor taking a CommandLine.
     *
     * @param commandLine The command line.
     */    
    public
    DeploymentApplication(CommandLine commandLine)
    {
        // Calls the default ApplicationBase constructor.
        super(commandLine);

        // Creates the associated deployer.
        setDeployer(new DeploymentProcess());
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    private void
    computeArgs(String args[])
    {
        for(int i=0;i<args.length;i++){
            if(args[i].length()>3&&
               args[i].substring(args[i].length()-3)
               .compareToIgnoreCase("zip")==0)
                this.zipFile=args[i];
            else  if(args[i].equals("-F"))
                this.distributedDeployment=true;
            else  if(args[i].startsWith("-F")){
                this.assemblyFactoryName=
                    System.getProperty("Assembly_factory_ns_dir")+args[i].substring(2);        
                this.distributedDeployment=true;
            }
        }           
    }    

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.api.Identifiable
    //
    // ==================================================================

    /**
     * Obtains its identity.
     *
     * @return Its identity.
     */
    public String
    getIdentity()
    {
        return "ccm_deploy"
             + ' ' + org.objectweb.ccm.util.ReleaseInfo.VERSION_MAJOR
             + '.' + org.objectweb.ccm.util.ReleaseInfo.VERSION_MINOR;
    }

    // ==================================================================
    //
    // Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    /**
     * Executes the main function, i.e.
     * 1 - Initializes the CORBA::ORB,
     * 2 - Calls super.runMain().
     *
     * @param args The command line arguments.
     */
    public void
    runMain(String[] args)
    {
        try
        {
            // Initializes the CORBA::ORB singleton.
            // Initializes the CORBA::ORB singleton.
            args = org.objectweb.ccm.CORBA.TheORB.initialize(args);
            org.objectweb.ccm.Components.Runtime.init(org.objectweb.ccm.CORBA.TheORB.getORB());
        }
        // If any exception thrown then reports it.
        //
        catch(org.objectweb.util.api.ExceptionWrapper exc)
        {
            report_exception(exc.getException());
        }
        catch(Exception exc)
        { 
            report_exception(exc);
        }

        // Calls ApplicationBase.runMain method.
        super.runMain(args);
    }

    /**
     * Starts the application.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     */
    public int
    start(String[] args)
    {
        
        computeArgs(args);
        boolean result = deploy(zipFile);
        return result?0:-1;
    }

    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtains the deployer.
     *
     * @return The deployer.
     */
    public DeploymentProcess
    getDeployer()
    {
        return this._deployer;
    }

    /**
     * Sets the deployer.
     *
     * @param deployer The deployer.
     */
    public void
    setDeployer(DeploymentProcess deployer)
    {
        _deployer = deployer;
        _deployer
            .setConsole(
                getConsole()
                );
    }

    /**
     * Deploys an XML Component Assembly Descriptor.
     *
     * @param assembly_filename The XML Component Assembly Descriptor file name.
     *
     * @return True if the deployment successes, otherwise false.
     */
    public boolean
    deploy(String zip_file)
    {
        
        boolean result=true;
            
        getConsole()
            .message(
                "Initiating deployment sequence of "
                + zip_file
                );
        
        org.omg.Components.Deployment.Assembly ass;
        
        try{

            try{
                
                if(distributedDeployment){
                    org.omg.Components.Deployment.AssemblyFactory af ;
                        
                    if(assemblyFactoryName!=null){
                        
                        org.omg.CosNaming.NamingContextExt context=
                            org.omg.CosNaming.NamingContextExtHelper
                            .narrow(org.objectweb.ccm.CORBA.TheNameService
                                    .getNamingContext()
                                    .getNamingContext());
                        
                        
                        af= org.objectweb.openccm
                            .OpenCCM_DCI.AssemblyFactoryManagerHelper
                            .narrow(context.resolve(context.to_name(assemblyFactoryName)))
                            .provide_assembly_factory();       
                    }                              
                    else {
                        
                        org.objectweb.openccm.OpenCCM_DCI.DCIManager
                            DCI=
                            org.objectweb.openccm.
                            OpenCCM_DCI.DCIManagerHelper.
                            narrow(org.objectweb.ccm.CORBA
                                   .TheORB.resolve_initial_reference("DCIManager"));
                        af=DCI.get_connection_assembly_factory();
                        
                    }
                    
                    if(af==null)
                        throw new org.omg.CosNaming.NamingContextPackage.NotFound();
                        
                    //Add the use of an Http server to provide the assemblyfile
                    java.net.ServerSocket s = new java.net.ServerSocket(0); 
                    
        
                    //System.out.println(s);        

                        
                    System.err.println("http://" + 
                                       java.net.InetAddress.getLocalHost().getHostAddress() + ':' +
                                       s.getLocalPort() + '/' +zip_file);
                                       
                    
                    //Thread serverThread=
                    (new Thread(new MicroServerHttp(s))).start();
                    
                    
                    
                    org.omg.Components.Cookie cok=
                        af.create_assembly("http://" + 
                                           java.net.InetAddress.getLocalHost().getHostAddress() + ':' +
                                           s.getLocalPort() + '/' +zip_file);
                    
                    ass=af.lookup(cok);
                    ass.build();  
                    //ass.setConsole(getConsole());  
                }
                else{
                    System.err.println
                        ("Proceed a local deployment, ");
                    (new org.objectweb.ccm.Deployment.AssemblyImpl(zip_file))
                        .build();
                }
            }
            catch(java.io.IOException ioe)
              {
                    System.err.println
                        ("Cannot use assembly factory to download file, proceed a local deployment, ");
                    System.err.println
                        ("the tear_down function will be disable from console");
                  
                    (new org.objectweb.ccm.Deployment.AssemblyImpl(zip_file))
                         .build();
                   
                    
                }  
            /*catch(java.net.UnknownHostException uoe)
              {
                    System.err.println
                        ("Cannot use assembly factory to download file, process a local deployment, ");
                    System.err.println
                        ("the tear_down function will be disable from console");
                  
                    (new org.objectweb.ccm.Deployment.AssemblyImpl(assembly_filename))
                         .build();
                   
                    
                         }  */
            catch( org.omg.CosNaming.NamingContextPackage.NotFound nf) 
                {
                    System.err.println
                        ("Cannot found an valid assembly factory, proceed a local deployment,");
                    System.err.println
                        ("the tear_down function will be disable from console");
                  
                    (new org.objectweb.ccm.Deployment.AssemblyImpl(zip_file))
                         .build();                   
                    
                }  
            catch(org.omg.CosNaming.NamingContextPackage.CannotProceed cp)
                {
                    System.err.println
                        ("Cannot use name service to found an valid assembly factory, proceed a local deployment,");
                    System.err.println
                        ("the tear_down function will be disable from console");                  
                    (new org.objectweb.ccm.Deployment.AssemblyImpl(zip_file))
                        .build();               
                }  
            catch(org.omg.CosNaming.NamingContextPackage.InvalidName in)
                {
                    System.err.println
                        ("Cannot found an valid assembly factory, proceed a local deployment,");
                    System.err.println
                        ("the tear_down function will be disable from console");                  
                    (new org.objectweb.ccm.Deployment.AssemblyImpl(zip_file))
                        .build();
                    
                }  
            catch(org.omg.Components.Deployment.InvalidAssembly IA)
                {
                    System.err.println
                        ("Internal error occured during retrieving of assembly, cannot process deployment");                
                    return false;
                }          
         
            //ass.build();      

        }
        catch(org.omg.Components.Deployment.InvalidLocation il)
            {
                System.err.println
                    ("Cannot found the CAD file, check your path and restart the deployment");
                result=false;
            }
        catch(org.omg.Components.CreateFailure cf)
            {
                System.err.println
                    ("Cannot achieved assembly deployment, be carefull only a partial assembly");
                System.err.println
                    ("is deployed, assembly behavior may be corrupt");
                result=false;
            }    
                    
        if (!result)
        {
            getConsole()
                .error(
                    "Sequence aborted during deployment of "
                    + zip_file
                    );
            return false;
        }

        getConsole()
            .message(
                "Successfully deployed "
                + zip_file
                );

        return result;
    }

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     */
    public static void
    main(String[] args)
    {
        DeploymentApplication application = new DeploymentApplication();
        application.runMain(args);
    }
}
