/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): Sylvain Leblanc____________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm;

//Importation of the needed packages
import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;

/**
 * A connectinterfaceDeployer to establish interface connections.
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */

public class ConnectinterfaceDeployer {

  
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**Internal exception to warn of impossibility to found a HomeInterface*/
    private class CannotResolveHomeInterfaceException extends Exception {};
    //Internal exception to warn of impossibility to resovle a reference
    private class CannotResolveReferenceException extends Exception{};  
    /** Cookie whitch store the connection information. */

    private org.omg.Components.Cookie cookie; 
    /** Reference of the user*/
    private org.omg.Components.CCMObject user;     
    


    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    
    /**
     * Resolve and existing interface and return it like the consumed object.
     * If any interface can be found an exception is thrown
     *
     * @param deployTable the internal table use to store information.
     * @param ei the existing interface to retrieve
     */
    private org.omg.CORBA.Object 
    proceedExistingInterface(java.util.Hashtable deployTable,
                             Existinginterface ei)
    throws Exception
    {
        return resolveReference(deployTable,ei.getFindby());    
    }
    
    
    /**
     * Internal method to resolve and retrieve the supportedinterface. 
     * First it try to find supportedinterface reference with 
     * the componentinstantiationref, if it is null
     * it try to resolve the reference with registration services.
     * If the both way failed an exception is thrown.
     *
     * @param deployTable the internal table use to store information.
     * @param csi the componentsupportedinterface to retrieve
     * @return the componentsupported interface
     */
    private org.omg.Components.CCMObject 
    proceedComponentSupportedInterface(java.util.Hashtable deployTable,
                                       Componentsupportedinterface csi)
    throws Exception
    {
        if(csi.getComponentinstantiationref()!=null)
            return (org.omg.Components.CCMObject)
                deployTable.get(csi.getComponentinstantiationref().getIdref());
        return org.omg.Components.
               CCMObjectHelper.narrow(resolveReference(deployTable,csi.getFindby()));        
    }
    
    /**
     *Internal method to resolve and retrieve the user.
     * First it try to find usesport reference with 
     * the componentinstantiationref, if it is null
     * it try to resolve the reference with registration services.
     * If the both way failed an exception is thrown.
     *
     * @param deployTable the internal table use to store information.
     * @param upi the usesports to retirves
     * @return The components using the connection
     */
    private org.omg.Components.CCMObject
    proceedUsesport (java.util.Hashtable deployTable,
                     Usesport upi)
    throws Exception
    {
        if(upi.getComponentinstantiationref()!=null)
            return (org.omg.Components.CCMObject)
                deployTable.get(upi.getComponentinstantiationref().getIdref());
        return org.omg.Components.
            CCMObjectHelper.narrow(resolveReference(deployTable,upi.getFindby()));
    }
    
    /**
     * Internal method to resolve and retrieve the providedobject.
     * First it try to resolve the provider reference with componentinstantiationref.
     * If it is null then it try to resolve the provider reference with 
     * registration services.
     * If the both way failed an exception is thrown.
     * If the provider is found the function return the providedobject invoking the
     * provided_facet on the provider with providesidentifier to parameter
     *
     * @param deployTable the internal table use to store information.
     * @param ppi the providesports to retrieves
     * @return The interface provides
     */
    private org.omg.CORBA.Object 
    proceedProvidesport (java.util.Hashtable deployTable , 
                         Providesport ppi)
    throws Exception
    {
        if(ppi.getComponentinstantiationref()!=null)
            return ((org.omg.Components.CCMObject)
                    (org.omg.CORBA.Object )deployTable.
                    get(ppi.getComponentinstantiationref().getIdref()))
                .provide_facet(ppi.getProvidesidentifier().getValue());
        
        return org.omg.Components.
            CCMObjectHelper.narrow(resolveReference(deployTable,ppi.getFindby()))
            .provide_facet(ppi.getProvidesidentifier().getValue());    
    }
    
    /**
     * Internal method to resolve and retrieve the Homeinterface.
     * First it try to resolve the reference with componentinstantiationref.
     * If it is null then it try to resolve the reference with registration services.
     * If the both way failed it try to found the interface with the homefinder service.
     * If all attempt failled an CannotResolveHomeInterfaceException is raised
     * If an HomeInterface is found it's return.
     *
     * @param deployTable the internal table use to store information.
     * @param hii the home interface to retrieves
     * @return The home interface found
     */
    private org.omg.Components.CCMHome 
    proceedHomeInterface (java.util.Hashtable deployTable ,
                          Homeinterface hii)
    throws Exception
    {
        /*try to resolve with homeplacementref*/
        if(hii.getHomeplacementref()!=null)
            return (org.omg.Components.CCMHome)
                deployTable.get(hii.getHomeplacementref().getIdref());
        
        if(hii.getFindby()!=null)
            try{//try to resolve reference with registration services
                return org.omg.Components.
                    CCMHomeHelper.narrow(resolveReference(deployTable,hii.getFindby()));
            }
            catch(Exception e){
                // search the reference with the homefinder
                //return proceedWithHomeFinder(hii.getFindby().getHomefinder());
            }
        // cannot found the interface raise an exception
        throw new CannotResolveHomeInterfaceException();
    }   
    
    /**
     * Internal method allowing to resolve the Findby tag using
     * the extensio manager tool. 
     *
     * @param deployTable  the internal table used to store information.
     * @param fb The findby information to process 
     */       
    private org.omg.CORBA.Object 
    resolveReference(java.util.Hashtable deployTable,
                     Findby fb)
    throws Exception
    {

        return  ((FindbyDeployer)
                 ((FindbyBean)fb)
                 .getExtensionManager()
                 .getExtensionByName("ccm.Deployer")
                 .getInstance())
            .resolveReference(deployTable,fb);

    }



    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================


    /**

     * Establish the connectinterface connection:
     * <ol>
     *		<li>Retrieve the consumedObject,</li>
     *		<li>Retrieve the user user,</li>
     *		<li>Connect the user with the provided object.</li>
     * </ol>
     *
     *	All Providesport , Homeinterface, Componentsupportedinterface
     *	and existing interface are supported, but only one can be
     *	defined only the first will be proceed.  If one of
     *	communicating part can't be found the connection can be
     *	establish the process is aborted, informing of the problem.
     */

    public void deploy(java.util.Hashtable deployTable,
                       Connectinterface cii)
    {
         try{
             System.out.println("Etablishing connection");
             org.omg.CORBA.Object providedObject=null ;
             /*Proceed a Providesport tag*/
             if(cii.getProvidesport()!=null){
                 providedObject = proceedProvidesport(deployTable,cii.getProvidesport());
             }/*Proceed the homeinterface tag*/
             else if(cii.getHomeinterface()!=null){
                 providedObject =proceedHomeInterface(deployTable,cii.getHomeinterface());    
             }/*Proceed the component supported interface tag*/
             else if(cii.getComponentsupportedinterface()!=null){
                 providedObject= 
                     proceedComponentSupportedInterface(deployTable,
                                                        cii.getComponentsupportedinterface());
             }/*proceed the existing interface tag*/
             else if(cii.getExistinginterface()!=null){
                 providedObject= proceedExistingInterface(deployTable,cii.getExistinginterface());
             }                 
             if(providedObject==null)
                 throw new Exception("Cannot establishes connection");
             
             /*get the uses object*/
             user=proceedUsesport(deployTable,cii.getUsesport());
             
             if(user==null)
                 throw new Exception("Cannot establishes connection invalid use port:"+cii.getUsesport().
                                     getUsesidentifier().getValue());
             
             /*org.omg.Components.ReceptacleDescription [] desc=user.get_all_receptacles();
               for(int i=0; i<desc.length;i++)
               System.err.println("Recepacle:"+i+"  "+desc[i].name);
               // call the connect method
               System.out.println("Nom du port :"+cii.getUsesport().getUsesidentifier().getValue());
               System.err.println("Connection to establish :"+ cii.getUsesport().
               getUsesidentifier().
               getValue());
             */
             cookie=user.connect(cii.getUsesport().
                                 getUsesidentifier().
                                 getValue(),providedObject);    
         }
         catch(Exception e)
             {
                 System.out.println("Cannot establish the <"+cii.getId()+"> connectinterface connection");
                 e.printStackTrace();
             }
    }
    
    /**
     * Break the connectinterface.
     *
     * @param deployTable the internal table use to store information.
     * @param cii the connectinterface to deploy    
     */ 
    public void 
    tearDown(java.util.Hashtable deployTable,
             Connectinterface cii)
    {
        try
            {
                // If the connection wasn't established stop the process
                if(user==null) return ;
                user.disconnect(cii.getUsesport().getUsesidentifier().getValue(),cookie);
            }
        catch(Exception e){e.printStackTrace();}

    } 

}
