/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc
Contributor(s): _____________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm;

//Importation of the needed packages
import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;

import org.objectweb.ccm.CORBA.TheTradingService;
import org.objectweb.openccm.services.trading.TraderTypeCodeUtils;

/**
 * Allows to compute actions on trading service from data 
 * contained in CCM descriptors.
 *
 * Provided features: 
 * <ul>
 *  <li>Registration of a home or component in the trading service.</li>
 * </ul>
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 *
 * @version 0.1
 */
public class RegisterwithtraderDeployer 
{

    private static final String header = "Trading Service problem: ";

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The CosTrading::OfferId provided by the trading registration. */
    protected String _oid;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     *  The default constructor.
     */
    public
    RegisterwithtraderDeployer ()
    {
        // Calling the super constructor
        super();

        // The registered offer id in the trader.
        this._oid = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Resolve the register interface of a named trader.
     *
     * @param name The path name of the trader to access.  The given
     * name must stands for a path from the starting trader to the
     * desired trader following a sequence of links.  Here we assume
     * that the path has the following form:
     * <code>link_name1/link_name2/link_nameN</code>.
     *
     * @return The register interface of the named trader, or the
     * domain local trader register interface if no name have been
     * provided, or null if the name doesn't exist or if the named
     * trader doesn't allows to export offers.
     */
    protected org.omg.CosTrading.Register 
    resolveExternalTrader(String name)
        throws org.omg.CosTrading.RegisterPackage.IllegalTraderName,
        org.omg.CosTrading.RegisterPackage.UnknownTraderName,
        org.omg.CosTrading.RegisterPackage.RegisterNotSupported
    {
        // Resolving the used trader
        org.omg.CosTrading.Register iRegister = null;
        iRegister = TheTradingService.getRegister();
        if (iRegister == null) return null;
        
        if (name != null) 
        {
            // Splitting the given name in a string array
            java.util.LinkedList path_list = new java.util.LinkedList();
            int idx = 0;
            int prev_idx = 0;
            idx = name.indexOf("/");
            while (idx != -1) {
                String current = name.substring(prev_idx, idx);
                if (! current.trim().equals("")) path_list.add(current);
                prev_idx = idx+1;
                idx = name.indexOf("/", prev_idx);
            }  
            String current = name.substring(prev_idx);
            if (! current.trim().equals("")) path_list.add(current);
            String[] path = (String[])path_list.toArray(new String[0]);

            // Making registration 
            iRegister = iRegister.resolve(path);
        }
        return iRegister;
    }

    /**
     * Construct a CosTrading property for a given service type using
     * the property name and value as a string. The Property value
     * type is checked using the service type repository description
     * facilities. Here, the <code>fully_describe_type</code> method
     * of the <code>CosTrading::Register</code> interface is used in
     * order to retrieve this information.
     *
     * @param name The name of the property to create.
     * @param value The value of the property as a string.
     *
     * @return The CosTrading property or null if cannot be constructed.
     *
     * @exception org.omg.CosTrading.IllegalServiceType If the service
     * type name is not valid.
     * @exception org.omg.CosTrading.UnknownServiceType If the service
     * type doesn't exist in the service type repository.
     */
    protected org.omg.CosTrading.Property 
    constructProperty(String name, String value, String servicetype)
        throws org.omg.CosTrading.IllegalServiceType,
        org.omg.CosTrading.UnknownServiceType
    {
        org.omg.CORBA.TypeCode tc = null;

        // Accessing to the service type repository in order to 
        // obtain the property type.
        org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.TypeStruct desc;
        desc = TheTradingService.getRepository().fully_describe_type(servicetype);
        for (int i=0 ; i<desc.props.length ; i++) 
        {
            if (desc.props[i].name.equals(name)) 
            {
                tc = desc.props[i].value_type;
                break;
            }
        }
        if (tc == null) return null;

        // Creating the used any value
        org.omg.CORBA.Any the_any = TraderTypeCodeUtils.insertInAny(tc, value);
        if (the_any == null) {
            return null;
        }

        org.omg.CosTrading.Property prop = new org.omg.CosTrading.Property();
        prop.name = name;
        prop.value = the_any;
        return prop;
    }

    /**
     * Remove a service offer from the OMG Trading Service.
     *
     * @param offerId The identifier of the offer to remove.
     */
    protected void
    remove_from_trader(String offerId)
    {
        try {
            TheTradingService.getRegister().withdraw(offerId);
        } catch (org.omg.CosTrading.IllegalOfferId ex) {
            System.err.println(RegisterwithtraderDeployer.header + 
                             "cannot withdraw offer '" + offerId + "'.");
        } catch (org.omg.CosTrading.UnknownOfferId ex) {
            System.err.println(RegisterwithtraderDeployer.header + 
                             "cannot withdraw offer '" + offerId + "'.");
        } catch (org.omg.CosTrading.RegisterPackage.ProxyOfferId ex) {
            System.err.println(RegisterwithtraderDeployer.header + 
                             "cannot withdraw offer '" + offerId + "'.");
        }
    }

    /**
     * Register a CORBA Object in the OMG Trading Service if available.
     *
     * @param desc CCM Assembly Descriptor information on trader registration.
     * @param to_register The CORBA Object to register in the trading service.
     */     
    protected String 
    register_in_trader(Registerwithtrader desc, 
                       org.omg.CORBA.Object to_register)
    {        
        String trader_name = desc.getTradername();
        org.omg.CosTrading.Register registerer = null;
        try {
            registerer = resolveExternalTrader(trader_name);
        } catch (org.omg.CosTrading.RegisterPackage.IllegalTraderName ex) {
            System.err.println(RegisterwithtraderDeployer.header + 
                             "cannot register in trader " + trader_name + ".");
            return null;
        } catch (org.omg.CosTrading.RegisterPackage.UnknownTraderName ex) {
            System.err.println(RegisterwithtraderDeployer.header + 
                             "cannot register in trader " + trader_name + ".");
            return null;
        } catch (org.omg.CosTrading.RegisterPackage.RegisterNotSupported ex) {
            System.err.println(RegisterwithtraderDeployer.header + 
                             "cannot register in trader " + trader_name + ".");
            return null;
        }
        if (registerer == null) {
            System.err.println(RegisterwithtraderDeployer.header + 
                             "no trading service found.");
            return null;
        }
        
        // Retrieves the service type name. 
        Traderexport export_desc = desc.getTraderexport();
        String st = export_desc.getTraderservicetypename().getValue();

        // Constructing CORBA properties
        java.util.List properties_desc = export_desc.getTraderproperties().getTraderpropertyList();
        org.omg.CosTrading.Property[] corba_props = new org.omg.CosTrading.Property[properties_desc.size()];
        int i = 0;
        for (java.util.Iterator iter = properties_desc.iterator() ; 
             iter.hasNext() ; i++)
        {
            // Values from descriptor
            Traderproperty xml_prop = (Traderproperty)iter.next();
            String p_name = xml_prop.getTraderpropertyname().getValue();
            String p_value = xml_prop.getTraderpropertyvalue().getValue();

            // OMG IDL values
            org.omg.CosTrading.Property the_prop = null;
            try {
                the_prop = constructProperty(p_name, p_value, st);
            } catch (org.omg.CosTrading.IllegalServiceType ex) {
                System.err.println(RegisterwithtraderDeployer.header + 
                                 "invalid service type.");
                return null;
            } catch (org.omg.CosTrading.UnknownServiceType ex) {
                System.err.println(RegisterwithtraderDeployer.header + 
                                 "invalid service type.");
                return null;
            }
            if (the_prop == null) {
                System.err.println(RegisterwithtraderDeployer.header + 
                                 "cannot construct property.");
                return null;
            } 
            corba_props[i] = the_prop;
        }

        // Proceeding with registration
        try {
            return registerer.export(to_register,st,corba_props);
        } catch (org.omg.CosTrading.RegisterPackage.InvalidObjectRef ex) {
            System.err.println(RegisterwithtraderDeployer.header + 
                             "Invalid CORBA Object reference.");
            return null;
        } catch (org.omg.CosTrading.IllegalServiceType ex) {
            System.err.println(RegisterwithtraderDeployer.header + 
                             "Bad service type \"" + ex.type + "\"." );
            return null;
        } catch (org.omg.CosTrading.UnknownServiceType ex) {
            System.err.println(RegisterwithtraderDeployer.header + 
                             "Bad service type \"" + ex.type + "\"." );
            return null;
        } catch (org.omg.CosTrading.RegisterPackage.InterfaceTypeMismatch ex) {
            System.err.println(RegisterwithtraderDeployer.header + 
                             "Object interface " + 
                             "mismatch with service type \"" + 
                             ex.type + 
                             "\"." );
            return null;
        } catch (org.omg.CosTrading.IllegalPropertyName ex) {
            System.err.println(RegisterwithtraderDeployer.header + 
                             "Bad property name \"" + ex.name + "\"." );
            return null;
        } catch (org.omg.CosTrading.PropertyTypeMismatch ex) {
            System.err.println(RegisterwithtraderDeployer.header + 
                             "Bad property type of \"" + 
                             ex.prop.name + 
                             "\" in service type \"" + 
                             ex.type + 
                             "\"." );
            return null;
        } catch (org.omg.CosTrading.ReadonlyDynamicProperty ex) {
            System.err.println(RegisterwithtraderDeployer.header + 
                             "Bad property of \"" + 
                             ex.name + 
                             "\" in service type \"" + 
                             ex.type + 
                             "\"." );
            return null;
        } catch (org.omg.CosTrading.MissingMandatoryProperty ex) {
            System.err.println(RegisterwithtraderDeployer.header + 
                             "Require property \"" + 
                             ex.name + 
                             "\" in service type \"" + 
                             ex.type + 
                             "\"." );
            return null;
        } catch (org.omg.CosTrading.DuplicatePropertyName ex) {
            System.err.println(RegisterwithtraderDeployer.header + 
                             "Duplicate property \"" + 
                             ex.name + 
                             "\"." );
            return null;
        }
    }    

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    public void 
    deploy(org.omg.CORBA.Object objRef, Registerwithtrader reg)
    {
        this._oid = register_in_trader(reg, objRef);
    }
    
    public void 
    tearDown(Registerwithtrader reg)
    {
        if (this._oid != null) remove_from_trader(this._oid);
        this._oid = null;
    }
}
