/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

/** Used to access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** Used to access AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST DelegationList. */
import org.objectweb.corba.ast.api.DelegationList;

/** To access AST SegmentDecl. */
import org.objectweb.corba.ast.api.SegmentDecl;

/** To access AST InterfaceDecl. */
import org.objectweb.corba.ast.api.InterfaceDecl;

/** To access AST CategoryKind. */
import org.objectweb.corba.ast.api.CategoryKind;

/** To use ArrayList. */
import java.util.ArrayList;

/**
 * ExecutorDeclImpl is a wrapper class for CIDL executor declarations.
 *
 *
 * Inherits from:
 * - CidlScopeImpl as executors are also IDL scopes.
 * - ExecutorDecl as OMG IDL for CIDL executor declarations.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class ExecutorDeclImpl
     extends CidlScopeImpl
  implements org.objectweb.corba.ast.api.ExecutorDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** List of feature delegations. */
    private DelegationListImpl feature_delegation_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the exception declaration.
     */
    public
    ExecutorDeclImpl(Repository rep,
                     ScopeImpl parent)
    {
        // Call the ScopeImpl constructor.
        super(rep, parent);

        // Init internal state.
        feature_delegation_ = new DelegationListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Declare CIF interfaces into Interface Repository.
     */
    protected void
    declareCIF( org.objectweb.corba.ast.api.ModuleDecl module,
                org.objectweb.corba.ast.api.NativeDecl ast)
    {
        org.objectweb.corba.ast.api.InterfaceList list = null;
        org.objectweb.corba.ast.api.OperationDecl op = null;
        org.objectweb.corba.ast.api.HomeExecutorDecl home_exec = null;
        org.objectweb.corba.ast.api.ComponentDecl comp = null;
        Declaration[] segments = null;
        Declaration[] implemented_facets = null;
        InterfaceDecl tmp, base = null, base_internal = null, itf = null;
        CategoryKind kind = null;

        // Get the managed component
        home_exec = (org.objectweb.corba.ast.api.HomeExecutorDecl)getParent();
        comp = home_exec.getHomeType().getManagedComponent();

        /** Create the segment base interface **/
        base = module.declareLocalInterface("CIF_SegmentBase");
        base.create();

        /** Create the component executor interface **/
        itf = module.declareLocalInterface("CIF_"+getName());
          // Feed inherited interface list
        list = itf.getInheritedInterfaceList();
          // the CIF_SegmentBase interface
        list.add(base);
          // the ExecutorLocator interface
        tmp = (InterfaceDecl)getRepository().lookup("::Components::ExecutorLocator");
        list.add(tmp);
          // the <kind>Component interface
        kind = getCompositionKind();
        if ((kind==CategoryKind.ck_entity) || (kind==CategoryKind.ck_process))
        {
            tmp = (InterfaceDecl)getRepository().lookup("::Components::EntityComponent");
        }
        else
        {
            tmp = (InterfaceDecl)getRepository().lookup("::Components::SessionComponent");
        }
        list.add(tmp);

          // the CCM_<component_name>_Executor interface (segmented strategy)
        tmp = comp.getLocalMainMapping();
        list.add(tmp);
          // Compute Implemented facets and sinks
        java.util.Set itf_set = new java.util.HashSet();
        implemented_facets = getMainSegmentFacets(comp);
        for (int i=0; i<implemented_facets.length; i++)
        {
            try {
                org.objectweb.corba.ast.api.InterfacePortDecl itf_port = null;
                itf_port = (org.objectweb.corba.ast.api.InterfacePortDecl)implemented_facets[i];
                itf_set.add( itf_port.getLocalMapping() );
            }catch(ClassCastException e){
                org.objectweb.corba.ast.api.EventPortDecl event_port = null;
                event_port = (org.objectweb.corba.ast.api.EventPortDecl)implemented_facets[i];
                itf_set.add( event_port.getEvent().getLocalConsumerMapping() );
            }
        }
        java.util.Iterator it = itf_set.iterator();
        while ( it.hasNext() )
        {
            list.add((InterfaceDecl) it.next());
        }

        itf.create();

        if (ast != null)
        {
            // create the get_state operation
            op = itf.startOperation("get_state");
            op.setType(ast);
            op.create();
        }


        /** Finish creation of the segment base interface **/
        op = base.startOperation("get_context");
          // Retrieve CCM_<component_name>_Context interface
        op.setType(comp.getLocalContextMapping());
        op.create();
        op = base.startOperation("get_main_segment");
        op.setType(itf);
        op.create();

        /** Call mapping for segments **/
        segments = getContents(true, DeclarationKind.dk_segment);
        for (int i=0; i<segments.length; i++)
        {
            ((org.objectweb.corba.ast.lib.SegmentDeclImpl)segments[i]).declareCIF( module, base, comp );
        }
    }

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 ExceptionDef.
     *
     * @param contained The ExceptionDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
      // TODO
    }

    /**
     * Loads infos of the CORBA 3.0 ExceptionDef.
     *
     * @param contained The ExceptionDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
      // TODO
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the exception
     *         declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return null;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Container reference.
     *
     * @return The Container object associated with the exception
     *          declaration.
     */
    protected org.omg.CORBA.Container
    getContainer()
    {
        return null;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain the associated composition category.
     *
     * @return The associated composition category.
     */
    public CategoryKind
    getCompositionKind()
    {
        return ((org.objectweb.corba.ast.api.CompositionDecl)getParent().getParent()).getCategoryKind();
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     *       as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        // TODO
        return null;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_executor;
    }

    /**
     * Close the exception scope.
     */
    public void
    create()
    {
        // Call the Cidl scope method
        super.create();

      // TODO check if the executor declaration is correct
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.ExecutorDecl
    //
    // ==================================================================

    /**
     * Obtain the feature delegation list.
     *
     * @return The feature delegation list.
     */
    public DelegationList
    getFeatureDelegationList()
    {
        return feature_delegation_;
    }

    /**
     * Start a new CIDL segment declaration.
     *
     * @param name The name of the CIDL segment declaration.
     *
     * @return The new created SegmentDecl.
     */
    public SegmentDecl
    startSegment(String name)
    {
        SegmentDecl decl = new SegmentDeclImpl(getRepository(), this);
        decl.setName(name);
        return decl;
    }

    /**
     * Get facets implemented directly by the main segment.
     *
     * @param comp - The managed component
     *
     * @return The facets (or sinks) list.
     */
    public Declaration[]
    getMainSegmentFacets(org.objectweb.corba.ast.api.ComponentDecl comp)
    {
        Declaration[] decls = null;
        ArrayList implemented_facets = new ArrayList();

        // Get ProvidesDecl and ConsumesDecl to build a list
        decls = ((org.objectweb.corba.ast.lib.ComponentDeclImpl)comp).getContents(
                        false, DeclarationKind.dk_provides + DeclarationKind.dk_consumes);
        for (int i=0; i<decls.length; i++)
        {
            implemented_facets.add( decls[i] );
        }

        // Get segments
        decls = getContents(true, DeclarationKind.dk_segment);

        // Compute Implemented facets list
        for (int i=0; i<decls.length; i++)
        {
            ((org.objectweb.corba.ast.lib.SegmentDeclImpl)decls[i]).
                    computeFacetsList(comp, implemented_facets);
        }
        return (Declaration[])implemented_facets.toArray(new Declaration[0]);
    }

}
