# ====================================================================
#
# OpenCCM: The Open CORBA Component Model Platform
# Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
# Contact: openccm@objectweb.org
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Pierre Carpentier.
# Contributor(s): ______________________________________.
#
# ====================================================================
#
#  This class provides methods to check all the constraints
#  defined in the UML Profile for CORBA (formal/02-04-01).
#
# ====================================================================

import ConstraintException;
import UMLOperations;

class CORBA2Checker
{

    # =================================================================
    # Empty constructor for the CORBA2Checker.
    # =================================================================
    proc __CORBA2Checker__(self)
    {
        self.umlOperations = UMLOperations.UMLOperations();
    }


    # =================================================================
    # Sets the UML Package in the UML repository.
    # umlPackage - The UML Package
    # =================================================================
    proc setUMLPackage (self, umlPackage)
    {
        self.umlPackage = umlPackage;
        self.umlOperations.setUMLPackage(umlPackage);
    }


    # =================================================================
    #   Prints an error message.
    #   stereotype - The stereotype which doesn't respect the constraint
    #   cause - The name of the element which causes the exception
    #   section - The corresponding section of the specification
    #   message - The error message
    # =================================================================
    proc error(stereotype, cause, section, message) {
        throw ConstraintException.ConstraintException (stereotype, cause, "UML Profile for CORBA (formal/02-04-01)", section, message);
    }


    # =================================================================
    #   Tests if an interface is a local interface
    # =================================================================
    proc isLocalInterface (interface) {
        for stereotype in interface.stereotype() {
            if (stereotype.name() == "CORBAInterface") {
                for tag in stereotype.tagged_value() {
                    if (tag.type().tag_type() == "isLocal") {
                        return true;
                    }
                }
            }
        }
        return false;
    }


    # =================================================================
    #   3.5.1 - CORBA Basic Types
    # =================================================================
    proc checkCORBAPrimitive (self, element)
    {
        name = element.name();
        if (name != "short" && name != "long" && name != "long long" && name != "double" && name != "long double"
            && name != "unsigned short" && name != "unsigned long" && name != "unsigned long long" && name != "any"
            && name != "boolean" && name != "string" && name != "octet" && name != "void" && name != "char"
            && name != "wchar" && name != "float" && name != "wstring" && name != "typecode" && name != "native") {
            error("CORBAPrimitive", name, "3.5.1", "The type is not recognized.");
        }
    }


    # =================================================================
    #   3.5.2 - CORBA User-defined Types
    # =================================================================
    proc checkCORBAUserDefinedType (self, element)
    {
        if (!element._is_a(UML.Core.Classifier)) {
            error ("CORBAUserDefinedType", element.name(), "3.5.2", "The stereotyped element is not a Classifier.");
        }
        # [1a]
        for ownedEl in element.owned_element() {
            if (ownedEl.name() == element.name()) {
                error("CORBAUserDefinedType", ownedEl.name(), "3.5.2", "All model elements representing CORBA Namespaces may not directly contain another element of the same name.");
            }
        }
        # [1b]
        for attribute in self.umlOperations.allAttributes(element) {
            if (!self.umlOperations.isStereotyped(attribute, "CORBAConstant")) {
                try {
                    if (!self.umlOperations.isStereotyped(attribute.type(), "CORBAPrimitive")
                        || attribute.owner_scope() != UML.DataTypes.ScopeKind.sk_instance
                        || attribute.target_scope() != UML.DataTypes.ScopeKind.sk_instance
                        || attribute.changeability() != UML.DataTypes.ChangeableKind.ck_changeable) {
                        error("CORBAUserDefinedType", element.name(), "3.5.2", "All Attributes of a CORBAUserDefinedType-stereotyped Classifier that are not stereotyped CORBAConstant must be of a type that is stereotyped CORBAPrimitive, and for which the ownerScope is \"instance\", the targetScope is \"instance\", and the changeability is \"changeable\".");
                    }
                } catch (Reflective.NotSet notSet) {
                }    
            }
        }
        # [2]
        for asso in self.umlOperations.associations(element) {
            numberNavigable = 0;
            assoname = "";
            for connect in asso.connection() {
                assoname = assoname + connect.participant().name();
                if (connect.is_navigable())
                    numberNavigable = numberNavigable + 1;
            }
            if (asso.connection().length != 2 || numberNavigable != 1) {
                error("CORBAUserDefinedType", assoname, "3.5.2", "All Associations in which a CORBAUserDefinedType-stereotyped Classifier participates that have navigable opposite AssociationEnds must be binary and unidirectional.");
            }
        }
        # [3]
        for end in self.umlOperations.navigableOppositeEnds(element) {
            if (end.changeability() != UML.DataTypes.ChangeableKind.ck_changeable
                || end.aggregation() != UML.DataTypes.AggregationKind.ak_none
                || end.target_scope() != UML.DataTypes.ScopeKind.sk_instance
                || (self.umlOperations.isStereokinded(end.participant(), "CORBAUserDefinedType") && self.umlOperations.isStereotyped(end.participant(), "CORBAPrimitive"))) {
                error("CORBAUserDefinedType", end.name(), "3.5.2", "All navigable opposite AssociationEnds of a CORBAUserDefinedType-stereotyped Classifier must have changeability \"changeable\", aggregation \"none\", targetScope \"instance\", and a type that is stereotyped with a descendant of CORBAUserDefinedType or stereotyped CORBAPrimitive.");
            }
        }
        # [4]
        for end in self.umlOperations.nonNavigableNearEnds(element) {
            if (end.target_scope() != UML.DataTypes.ScopeKind.sk_instance) {
                error("CORBAUserDefinedType", end.name(), "3.5.2", "All non-navigable near AssociationEnds of a CORBAUserDefinedType-stereotyped Classifier must have targetScope \"instance\".");
            }
        }
        # [5]
        for opEnd in self.umlOperations.navigableOppositeEnds(element) {
            if (!self.umlOperations.isStereotyped(opEnd.participant(), "CORBAInterface")) {
                
                if (self.umlOperations.otherAssociationEnd(element, opEnd).aggregation() != UML.DataTypes.AggregationKind.ak_composite) {
                    error("CORBAUserDefinedType", opEnd.name(), "3.5.2", "All Associations in which a CORBAUserDefinedType-stereotyped Classifier participates that have a navigable opposite AssociationEnd whose type is not a CORBAInterface-stereotyped Classifier must have a near AssociationEnd with the aggregation \"composite.\"");
                }
            }
        }
        # [6]
        for assoc in self.umlOperations.associations(element) {
            if (assoc._is_a(UML.Core.UmlAssociationClass)) {
                error("CORBAUserDefinedType", assoc.name(), "3.5.2", "A CORBAUserDefinedType-stereotyped Classifier cannot participate in any AssociationClasses.");
            }
        }
    }


    # =================================================================
    #   3.5.3 - CORBA Structured Types
    # =================================================================
    proc checkCORBAStructuredType (self, element)
    {
        self.checkCORBAUserDefinedType(element);
        orderTags = array.create(0);
        for feat in element.feature() {
            for tag in feat.tagged_value() {
                if (tag.type().name() == "IDLOrder" || tag.type().name() == "CORBAIDLOrder")
                    orderTags.append(tag);
            }
        }
        for end in self.umlOperations.navigableOppositeEnds(element) {
            for tag in end.tagged_value() {
                if (tag.type().name() == "IDLOrder" || tag.type().name() == "CORBAIDLOrder")
                    orderTags.append(tag);
            }
        }
        for ownedElement in element.owned_element() {
            for ownedElement in feat.tagged_value() {
                if (tag.type().name() == "IDLOrder" || tag.type().name() == "CORBAIDLOrder")
                    orderTags.append(tag);
            }
        }
        orderValues = array.create(0);
        for tag in orderTags {
            orderValues.append(tag.data_value());
        }
        numOfOrderValues = orderTags.length;
        for valueSet in orderValues {
            for value in valueSet {
                valueLong = long._fromString(value);
                ## integers between 0 and numOfOrderValues AND each value is unique
                if ((valueLong < 0 || valueLong >= numOfOrderValues) || (orderValues.index(value) != orderValues.rindex(value))) {
                    error("CORBAStructuredType", element.name(), "3.5.3", "All Attributes, navigable opposite AssociationEnds, and ownedElements of a CORBAStructuredType-stereotyped Class must have a tagged value IDLOrder whose values are contiguous integers starting from 0.");
                }
            }
        }
    }


    # =================================================================
    #   3.5.4 - Module
    # =================================================================
    proc checkCORBAModule (self, element)
    {
        if (!element._is_a(UML.ModelManagement.Package)) {
            error("CORBAModule", element.name(), "3.5.4", "The stereotyped element is not a Package.");
        }
        # 3.5.2 - [1]
        for ownedEl in element.owned_element() {
            if (ownedEl.name() == element.name()) {
                error("CORBAModule", ownedEl.name(), "3.5.4", "All model elements representing CORBA Namespaces may not directly contain another element of the same name.");
            }
        }
        numberCORBAConstants = 0;
        for owned in element.owned_element() {
            # [1]
            if (!(self.umlOperations.isStereotyped(owned, "CORBAModule")
                  || self.umlOperations.isStereotyped(owned, "CORBAConstants")
                  || self.umlOperations.isStereokinded("CORBAUserDefinedType"))) {
                error("CORBAModule", element.name(), "3.5.4", "A CORBAModule-stereotyped package may directly contain only CORBAModule-stereotyped packages or Classes stereotyped as CORBAConstants or a descendant of CORBAUserDefinedType.");
            }
            # [2]
            if (self.umlOperations.isStereotyped(owned, "CORBAConstants")) {
                if (numberCORBAConstants == 1) {
                    error("CORBAModule", element.name(), "3.5.4", "A CORBAModule-stereotyped package may directly contain at most one Class stereotyped as CORBAConstants.");
                }
                numberCORBAConstants = numberCORBAConstants + 1;
            }
        }
    }


    # =================================================================
    #   3.5.5 - CORBA Object Types
    # =================================================================
    proc checkCORBAObjectType (self, object)
    {
        # [1]
        for feat in object.feature() {
            if (feat._is_a(UML.CommonBehavior.Reception)) {
                error("CORBAObjectType", object.name(), "3.5.5", "A CORBAObjectType-stereotyped Class may not have Receptions.");
            }
        }
        # [2]
        for ownedEl in object.owned_element() {
            if (!(self.umlOperations.isStereokinded(ownedEl, "CORBAConstructedType")
                || self.umlOperations.isStereokinded(ownedEl, "CORBAIndexedType")
                || self.umlOperations.isStereokinded(ownedEl, "CORBAWrapper")))
                error("CORBAObjectType", ownedEl.name(), "3.5.5", "The ownedElements of a CORBAObjectType-stereotyped Class may only be Classes stereotyped with a descendant of CORBAConstructedType, CORBAIndexedType, or CORBAWrapper.");
        }
        # [3]
        parentStereotype = 0;
        for generalization in object.generalization() {
            if (self.umlOperations.isStereotyped(generalization.parent(), "CORBAObjectType")
                || self.umlOperations.isStereotyped(generalization.parent(), "CORBAValue")
                || self.umlOperations.isStereotyped(generalization.parent(), "CORBACustomValue")
                || self.umlOperations.isStereotyped(generalization.parent(), "CORBAInterface")
                || self.umlOperations.isStereotyped(generalization.parent(), "CORBAHome")
                || self.umlOperations.isStereotyped(generalization.parent(), "CORBAComponent"))
                parentStereotype = parentStereotype + 1;
        }
        if (parentStereotype < object.generalization().length) {
            error("CORBAObjectType", object.name(), "3.5.5", "All parents of a CORBAObjectType-stereotyped Class must have the same stereotype as the Class.");
        }
    }


    # =================================================================
    #   3.5.6 - Interface
    # =================================================================
    proc checkCORBAInterface (self, element)
    {
        self.checkCORBAObjectType(element);
        # [1]
        for attrib in self.umlOperations.allAttributes(element) {
            if (attrib.visibility() != UML.DataTypes.VisibilityKind.vk_public) {
                error("CORBAInterface", element.name(), "3.5.6", "All Attributes of a CORBAInterface-stereotyped Class must have visibility \"public\".");
            }
        }
        # [2]
        for oppositeEnd in self.umlOperations.navigableOppositeEnds(element) {
            if (oppositeEnd.visibility() != UML.DataTypes.VisibilityKind.vk_public) {
                error("CORBAInterface", element.name(), "3.5.6", "All navigable opposite AssociationEnds of a CORBAInterface-stereotyped Class must have visibility \"public\".");
            }
        }
        # [3]
        if (isLocalInterface(element)) {
            for g in element.generalization() {
                if (!isLocalInterface(g.parent())) {
                    error("CORBAInterface", element.name(), "3.5.6", "A CORBAInterface-stereotyped Class tagged \"isLocal\" can only participate in Generalizations with other CORBAInterface-stereotyped Classes tagged \"isLocal\".");
                }
            }
        }
    }


    # =================================================================
    #   3.5.7 - Value Types - CORBAValue
    # =================================================================
    proc checkCORBAValue (self, element)
    {
        if (!element._is_a(UML.Core.UmlClass)) {
            error("CORBAValue", element.name(), "3.5.7", "The stereotyped element is not a Class.");
        }
        # [1]
        for attrib in self.umlOperations.allAttributes(element) {
            if (attrib.visibility() != UML.DataTypes.VisibilityKind.vk_public && attrib.visibility() != UML.DataTypes.VisibilityKind.vk_private) {
                error("CORBAValue", element.name(), "3.5.7", "All Attributes of a CORBAValue-stereotyped Class must have visibility \"public\" or \"private\".");
            }
        }
        # [2]
        for end in self.umlOperations.navigableOppositeEnds(element) {
            if (attrib.visibility() != UML.DataTypes.VisibilityKind.vk_public && attrib.visibility() != UML.DataTypes.VisibilityKind.vk_private)
                error("CORBAValue", element.name(), "3.5.7", "All navigable opposite AssociationEnds of a CORBAValue-stereotyped Class must have visibility \"public\" or \"private\".");
        }
        # [3]
        numberConcrete = 0;
        if (!element.is_abstract()) {
            for generalization in element.generalization() {
                if (self.umlOperations.isStereokinded(generalization.parent(), "CORBAValue") && !generalization.parent().is_abstract()) {
                    if (numberConcrete == 1) {
                        error("CORBAValue", element.name(), "3.5.7", "A concrete CORBAValue-stereotyped Class may only specialize a single other concrete CORBAValue-stereotyped Class.");
                    }
                    numberConcrete = numberConcrete + 1;
                }
            }
        }
        # [4]
        numberSupportedInterface = 0;
        for generalization in element.generalization() {
            if (self.umlOperations.isStereotyped(generalization.parent(), "CORBAInterface")) {
                numberSupportedInterface = numberSupportedInterface + 1;
                supportsGeneralization = generalization;
                if ((numberSupportedInterface == 1 && !self.umlOperations.isStereotyped(supportsGeneralization, "CORBAValueSupports"))
                    || numberSupportedInterface > 1)
                    error("CORBAValue", element.name(), "3.5.7", "A CORBAValue-stereotyped Class may only specialize a single CORBAInterface-stereotyped Class, and it must do so using a CORBAValueSupports-stereotyped Generalization.");
            }
        }
        # [5]
        numberValueFactory = 0;
        for operation in self.umlOperations.allOperations(element) {
            if (self.umlOperations.isStereotyped(operation, "CORBAValueFactory")) {
                if (numberValueFactory == 1) {
                    error("CORBAValue", element.name(), "3.5.7", "A CORBAValue-stereotyped Cass may only contain a single Operation stereotyped as CORBAValueFactory");
                }
                numberValueFactory = numberValueFactory + 1;
            }
        }
    }


    # =================================================================
    #   3.5.7 - Value Types - CORBACustomValue
    # =================================================================
    proc checkCORBACustomValue (self, element)
    {
        self.checkCORBAValue(element);
        # [1]
        for generalization in element.generalization() {
            if (generalization.parent() == element && self.umlOperations.isStereotyped(generalization, "CORBATruncatable")) {
                error("CORBACustomValue", element.name(), "3.5.7", "A CORBACustomValue-stereotyped Class may not be truncated by a CORBATrunctable-stereptyped Generalization that specializes the Class.");
            }
        }
    }


    # =================================================================
    #   3.5.7 - Value Types - CORBAValueSupports
    # =================================================================
    proc checkCORBAValueSupports (self, element)
    {
        if (!element._is_a(UML.Core.Generalization)) {
            error("CORBAValueSupports", element.name(), "3.5.7", "The stereotyped element is not a Generalization.");
        }
        # [1]
        if (!self.umlOperations.isStereotyped(element.parent(), "CORBAInterface") || !self.umlOperations.isStereokinded(element.child, "CORBAValue")) {
            error("CORBAValueSupports", element.name(), "3.5.7", "A CORBAValueSupports-stereotyped Generalization must have a CORBAInterface-stereotyped Class as its parent and a CORBAValue-stereotyped or CORBACustomValue-stereotyped Class as its child.");
        }
    }


    # =================================================================
    #   3.5.7 - Value Types - CORBATruncatable
    # =================================================================
    proc checkCORBATruncatable (self, element)
    {
        if (!element._is_a(UML.Core.Generalization)) {
            error("CORBATruncatable", element.name(), "3.5.7", "The stereotyped element is not a Generalization.");
        }
        # [1]
        if (!self.umlOperations.isStereokinded(element.parent(), "CORBAValue") || parent().is_abstract()
            || !self.umlOperations.isStereokinded(element.child, "CORBAValue") || child.is_abstract()) {
            error("CORBATruncatable", element.name(), "3.5.7", "A CORBATruncatable-stereotyped Generalization must have a concrete CORBAValue-steretyped or CORBACustomValue-stereotyped Class as its parent and has the same restriction as to its child.");
        }
    }


    # =================================================================
    #   3.5.7 - Value Types - CORBAValueFactory
    # =================================================================
    proc checkCORBAValueFactory (self, element)
    {
        if (!element._is_a(UML.Core.Operation)) {
            error("CORBAValueFactory", element.name(), "3.5.7", "The stereotyped element is not an Operation.");
        }
        # [1]
        for parameter in element.parameter() {
            if (param.kind() != UML.DataTypes.ParameterDirectionKind.pdk_in) {
                error("CORBAValueFactory", element.name(), "3.5.7", "A CORBAValueFactory-stereotyped Operation can have only in parameters and has no return type.");
            }
        }
        # [2]
        if (!self.umlOperations.isStereokinded(element.owner, "CORBAValue")) {
            error("CORBAValueFactory", element.name(), "3.5.7", "A CORBAValueFactory-stereotyped Operation must be owned by a CORBAValue-stereotyped or CORBACustomValue-stereotyped Class.");
        }
    }


    # =================================================================
    #   3.5.8 - CORBA Wrapper Types
    # =================================================================
    proc checkCORBAWrapper (self, element)
    {
        self.checkCORBAUserDefinedType(element);
        # [1]
        numberChild = 0;
        for gen in element.generalization() {
            if (gen.child() == element()) {
                if (numberChild == 1) {
                    error("CORBAWrapper", element.name(), "3.5.8", "A CORBAWrapper-stereotyped Classifier must participate as the child in exactly one Generalization relationship.");
                }
            }
            numberChild = numberChild + 1;
        }
        # [2]
        for gen in element.generalization() {
            if (!self.umlOperations.isStereotyped(gen.parent(), "CORBAPrimitive")
                && !self.umlOperations.isStereokinded(gen.parent(), "CORBAUserDefinedType")) {
                error("CORBAWrapper", element.name(), "3.5.8", "The parent of a CORBAWrapper-stereotyped Classifier must be stereotyped as CORBAPrimitive or as a descendant of CORBAUserDefinedType.");
            }
        }
        # [3]
        for gen in element.generalization() {
            try {
                if (!gen.discriminator() != "") {
                    gen.powertype(); # Test if a value is present, else NotSet Exception.
                    error("CORBAWrapper", element.name(), "3.5.8", "The Generalization relationship in which a CORBAWrapper-stereotyped Classifier participates has the empty string as its discriminator and no powertypes.");
                }
            } catch (Reflective.NotSet notSet) {}
        }
        # [4]
        if (element.feature().length > 0) {
            error("CORBAWrapper", element.name(), "3.5.8", "A CORBAWrapper-stereotyped Classifier may not have any non-inherited features.");
        }
        # [5]
        if (navigableOppositeEnds(element).length > 0) {
            error("CORBAWrapper", element.name(), "3.5.8", "A CORBAWrapper-stereotyped Classifier may not participate in any Associations with navigable opposite AssociationEnds.");
        }
        # [6]
        if (!element._is_a(UML.Core.DataType) && !element._is_a(UML.Core.UmlClass)) {
            error("CORBAWrapper", element.name(), "3.5.8", "A CORBAWrapper can only extend a DataType or a Class.");
        }
    }


    # =================================================================
    #   3.5.9 - Typedef
    # =================================================================
    proc checkCORBATypedef (self, element)
    {
        self.checkCORBAWrapper(element);
        # [1]
        for gen in element.generalization() {
            if (self.umlOperations.isStereotyped(gen.parent(), "CORBAAnonymousSequence")
                || self.umlOperations.isStereotyped(gen.parent(), "CORBAAnonymousArray")) {
                error("CORBATypedef", element.name(), "3.5.9", "The parent of a CORBATypedef-stereotyped Classifier must not be stereotyped as CORBAAnonymousSequence or CORBAAnonymousArray.");
            }
        }
    }


    # =================================================================
    #   3.5.10 - Boxed Value Types
    # =================================================================
    proc checkCORBABoxedValue (self, element)
    {
        self.checkCORBAWrapper(element);
    }


    # =================================================================
    #   3.5.11 - Constant Declaration - CORBAConstants
    # =================================================================
    proc checkCORBAConstants (self, element)
    {
        # [1]
        if (!self.umlOperations.isStereotyped(element.namespace(), "CORBAModule")) {
            error("CORBAConstants", element.name(), "3.5.11", "A CORBAConstants-stereotyped Class must be directly contained by a CORBAModule-stereotyped package.");
        }
        # [2]
        for feature in element.feature() {
            if (!feature._is_a(UML.Core.Attribute) || !self.umlOperations.isStereotyped(feature, "CORBAConstants")) {
                error("CORBAConstants", element.name(), "3.5.11", "All the features of a CORBAConstants-stereotyped Class must be CORBAConstant-stereotyped Attributes.");
            }
        }
        # [3]
        if (associations(element).length > 0) {
            error("CORBAConstants", element.name(), "3.5.11", "A CORBAConstants-stereotyped UtilityClass cannot participate in any Associations.");
        }
    }


    # =================================================================
    #   3.5.11 - Constant Declaration - CORBAConstant
    # =================================================================
    proc checkCORBAConstant (self, element)
    {
        # [1]
        if (element.changeability() != UML.DataTypes.ChangeableKind.ck_frozen || element.owner_scope() != UML.DataTypes.ScopeKind.sk_classifier) {
            error("CORBAConstant", element.name(), "3.5.11", "A CORBAConstant-stereotyped Attribute has the changeability \"frozen\" and the ownerScope \"classifier\".");
        }
        # [2]
        if (!self.umlOperations.isStereotyped(element.owner(), "CORBAConstants") && !self.umlOperations.isStereokinded(element.owner(), "CORBAObjectType")) {
            error("CORBAConstant", element.name(), "3.5.11", "The owner of a CORBAConstant-stereotyped Attribute must be stereotyped CORBAConstants or CORBAObjectType.");
        }
    }


    # =================================================================
    #   3.5.12 - Constructed Types
    # =================================================================
    proc checkCORBAConstructedType (self, element)
    {
        self.checkCORBAStructuredType(element);
        # [1]
        for feature in element.feature() {
            if (!feature._is_a(UML.Core.Attribute) || feature.visibility() != UML.DataTypes.VisibilityKind.vk_public) {
                error("CORBAConstructedType", element.name(), "3.5.11", "All features of a CORBAConstructedType-stereotyped Classifier must be Attributes with visibility \"public\".");
            }
        }
        # [2]
        for end in self.umlOperations.navigableOppositeEnds(element) {
            if (end.visibility() != UML.DataTypes.VisibilityKind.vk_public) {
                error("CORBAConstructedType", element.name(), "3.5.11", "All navigable opposite AssociationEnds of a CORBAConstructedType must have visibility \"public\".");
            }
        }
        # [3]
        for generalization in self.umlPackage.core_ref.generalization_ref.all_of_type_generalization {
            if (generalization.parent() == element || generalization.child() == element)
                error("CORBAConstructedType", element.name(), "3.5.11", "A CORBAConstructedType-stereotyped Classifier cannot participate in any Generalization relationships.");
        }
    }


    # =================================================================
    #   3.5.13 - CORBAStructType
    # =================================================================
    proc checkCORBAStructType (self, element)
    {
        self.checkCORBAConstructedType(element);
        # [1]
        for attrib in self.umlOperations.allAttributes(element) {
            if (attrib.multiplicity().range()[0].lower() != 1 || attrib.multiplicity().range()[0].upper() != 1) {
                error("CORBAStructType", element.name(), "3.5.13", "All the Attributes of a CORBAStructType-stereotyped Classifier must have multiplicity 1..1.");
            }
        }
        for end in self.umlOperations.navigableOppositeEnds(element) {
            # [2]
            if (end.multiplicity().range()[0].upper() != 1) {
                error("CORBAStructType", element.name(), "3.5.13", "All the navigable opposite AssociationEnds of a CORBAStructType-stereotyped Classifier must have the upper multiplicity value equal to 1.");
            }
            # [3]
            if (!self.umlOperations.isStereotyped(end, "CORBAInterface") && end.multiplicity().range()[0].lower() != 1) {
                error("CORBAStructType", element.name(), "3.5.13", "All the navigable opposite AssociationEnds of a CORBAStructType-stereotyped Classifier whose type is not a CORBAInterface-stereotyped Class must have the lower multiplicity value equal to 1.");
            }
        }
    }


    # =================================================================
    #   3.5.14 - CORBAStruct
    # =================================================================
    proc checkCORBAStruct (self, element)
    {
        self.checkCORBAStructType(element);
    }


    # =================================================================
    #   3.5.15 - Discriminated Union
    # =================================================================
    proc checkCORBAUnion (self, element)
    {
        self.checkCORBAConstructedType(element);
        # [1]
        numberSwitch = 0;
        otherElement = array.create(0);
        for attrib in self.umlOperations.allAttributes(element) {
            if (self.umlOperations.isStereotyped(attrib, "switch")) {
                switch = attrib;
                numberSwitch = numberSwitch + 1;
            } else {
                otherElement.append(attrib);
            }
        }
        for end in self.umlOperations.navigableOppositeEnds(element) {
            if (self.umlOperations.isStereotyped(end.participant(), "switchEnd") || self.umlOperations.isStereotyped(end, "switchEnd")) {
                switch = end;
                numberSwitch = numberSwitch + 1;
            } else {
                otherElement.append(end);
            }
        }
        if (numberSwitch != 1) {
            error("CORBAUnion", element.name(), "3.5.15", "Either exactly one of the Attributes or exactly one of the navigable opposite AssociationEnds of a CORBAUnion-stereotyped Class (but not both) must be stereotyped as switch (in the case of an Attribute) or switchEnd (in the case of AssociationEnd).");
        }
        # [2]
        if (switch.multiplicity().range()[0].lower() != 1 || switch.multiplicity().range()[0].upper() != 1) {
            error("CORBAUnion", element.name(), "3.5.15", "The Attribute or AssociationEnd that represents the switch of the IDL union represented by the CORBAUnion-stereotyped Class must have multiplicity 1..1.");
        }
        # [3]
        for other in otherElement {
            if ((!self.umlOperations.isStereotyped(other, "switch") || (!self.umlOperations.isStereotyped(other, "switchEnd")))
                 && (other.multiplicity().range()[0].lower() != 0 || other.multiplicity().range()[0].upper() != 1)) {
                error("CORBAUnion", element.name(), "3.5.15", "With the exception of the element representing the switch, every Attribute and navigable opposite AssociationEnd of a CORBAUnion-stereotyped Class must have the multiplicity 0..1 and a tagged value \"case\".");
            }
        }
    }


    # =================================================================
    #   3.5.16 - Enum
    # =================================================================
    proc checkCORBAEnum (self, element)
    {
        self.checkCORBAConstructedType(element);
        # [1]
        for attrib in self.umlOperations.allAttributes(element) {
            idlOrderValue = "notset";
            for tag in attrib.tagged_value() {
                if (tag.type().name() == "IDLOrder" || tag.type().name() == "CORBAIDLOrder")
                    idlOrderValue = tag.data_value()[0];
            }
            if (attrib.multiplicity().range()[0].lower() != 1 || attrib.multiplicity().range()[0].upper() != 1
                || attrib.initial_value().body() != idlOrderValue) {
                error("CORBAEnum", element.name(), "3.5.16", "All the Attributes of a CORBAEnum-stereotyped Class must have multiplicity 1..1, type CORBA::short and an initialValue equal to the value of its IDLOrder tag.");
            }
        }
        # [2]
        if (self.umlOperations.navigableOppositeEnds(element).length > 0) {
            error("CORBAEnum", element.name(), "3.5.16", "A CORBAEnum-stereotyped Class may not participate in any Association that has navigable opposite AssociationEnds.");
        }
    }


    # =================================================================
    #   3.5.17 - Exception
    # =================================================================
    proc checkCORBAException (self, element)
    {
        self.checkCORBAStructType(element);
        # [1]
        for end in self.umlOperations.allEnds(element) {
            if (end.participant() == element && end.is_navigable()) {
                error("CORBAException", element.name(), "3.5.17", "A CORBAException-stereotyped Exception cannot be the type of a navigable AssociationEnd.");
            }
        }
    }


    # =================================================================
    #   3.5.18 - Indexed Types
    # =================================================================
    proc checkCORBAIndexedType (self, element)
    {
        self.checkCORBAUserDefinedType(element);
        # [1]
        if (element.feature().length > 0) {
            error("CORBAIndexedTypes", element.name(), "3.5.18", "A CORBAIndexedType-stereotyped Class has no features.");
        }
        # [2]
        if (self.umlOperations.navigableOppositeEnds(element).length != 1) {
            error("CORBAIndexedTypes", element.name(), "3.5.18", "A CORBAIndexedType-stereotyped Class participates in exactly one Association that has a navigable opposite AssociationEnd.");
        }
        navigableOppositeEnd = self.umlOperations.navigableOppositeEnds(element)[0];
        # [3]
        if (navigableOppositeEnd.visibility() != UML.DataTypes.VisibilityKind.vk_public) {
            error("CORBAIndexedTypes", element.name(), "3.5.18", "The single navigable opposite AssociationEnd of a CORBAIndexedType-stereotyped Class must have the visibility \"public\".");
        }
        # [4a]
        if (self.umlOperations.nonNavigableNearEnds(element).length != 1) {
            error("CORBAIndexedTypes", element.name(), "3.5.18", "There is exactly one Association in which a CORBAIndexedType participates as the type of the near, non-navigable AssociationEnd.");
        }
        navigableNearEnd = self.umlOperations.nonNavigableNearEnds(element)[0];
        # [4b]
        # Not implemented because of the Constraint definition.
        # [5]
        if (self.umlOperations.isStereotyped(navigableOppositeEnd.participant(), "CORBAInterface") &&
            (self.umlOperations.navigableOppositeEnd.multiplicity().range()[0].lower() > 1
            || self.umlOperations.navigableOppositeEnd.multiplicity().range()[0].upper() != 1)) {
            error("CORBAIndexedTypes", element.name(), "3.5.18", "If the type being indexed is a CORBAInterface-stereotyped Class, then the opposite AssociationEnd of the one qualified Association in which the CORBAIndexedType-stereotyped Class participates has multiplicity lower bound of at most 1 and multiplicity upper bound of exactly 1.");
        }
        # [6]
        if (!self.umlOperations.isStereotyped(self.umlOperations.navigableOppositeEnd.participant(), "CORBAInterface") &&
            (self.umlOperations.navigableOppositeEnd.multiplicity().range().lower() != 1
            || self.umlOperations.navigableOppositeEnd.multiplicity().range().upper() != 1
            || self.umlOperations.navigableNearEnd.multiplicity().range().lower() != 1
            || self.umlOperations.navigableNearEnd.multiplicity().range().upper() != 1)) {
            error("CORBAIndexedTypes", element.name(), "3.5.18", "If the type being indexed is not a CORBAInterface-stereotyped Class, then \n the opposite AssociationEnd of the one qualified Association in which the CORBAIndexedType-stereotyped Class participates has multiplicity 1..1, and \n the near AssociationEnd of the qualified Association has multiplicity lower bound of at most 1 and upper bound of exactly 1.");
        }
        # [7]
        if (element.owned_element().length > 0) {
            error("CORBAIndexedTypes", element.name(), "3.5.18", "A CORBAIndexedType-stereotyped Class cannot have any ownedElements.");
        }
        # [8]
        for generalization in self.umlPackage.core_ref.generalization_ref.all_of_type_generalization {
            if (generalization.parent() == element || generalization.child() == element)
                error("CORBAIndexedTypes", element.name(), "3.5.18", "A CORBAIndexedType-stereotyped Class cannot participate in any Generalization relationships.");
        }
    }


    # =================================================================
    #   3.5.19 - Sequence - CORBASequence
    # =================================================================
    proc checkCORBASequence (self, element)
    {
        self.checkCORBAIndexedType(element);
        # [1]
        correctIndexName = false;
        if (self.umlOperations.nonNavigableNearEnds(element).length == 1 && self.umlOperations.nonNavigableNearEnds(element)[0].qualifier().length == 1) {
            if (self.umlOperations.nonNavigableNearEnds(element)[0].qualifier()[0].name() == "index")
                correctIndexName = true;
        }
        if (!correctIndexName) {
            error("CORBASequence", element.name(), "3.5.19", "The single non-navigable near AssociationEnd of a CORBASequence-stereotyped Class must have a single qualifier with the name \"index\".");
        }
        # [2a]
        # OCL not translated
        # [2b]
        # OCL not translated
        # [3]
        if (element.owned_element().length > 0) {
            error("CORBASequence", element.name(), "3.5.19", "A CORBAIndexedType-stereotyped Class cannot have any ownedElements.");
        }
    }


    # =================================================================
    #   3.5.19 - Sequence - CORBAAnonymousSequence
    # =================================================================
    proc checkCORBAAnonymousSequence (self, element)
    {
        self.checkCORBASequence(element);
        # [1]
        numberEnds = 0;
        for end in self.umlOperations.navigableOppositeEnds(element) {
            if (numberEnds == 1 || end.multiplicity().range()[0].lower() != 1 || end.multiplicity().range()[0].upper() != 1) {
                error("CORBAAnonymousSequence", element.name(), "3.5.19", "A CORBAAnonymousSequence-stereotyped Class must have exactly one navigable opposite AssociationEnd whose multiplicity is 1..1.");
            }
            numberEnds = numberEnds + 1;
        }
    }


    # =================================================================
    #   3.5.20 - Array - CORBAArray
    # =================================================================
    proc checkCORBAArray (self, element)
    {
        self.checkCORBAIndexedType(element);
        # [1]
        dimensions = self.umlOperations.nonNavigableNearEnds(element).length;
        for end in self.umlOperations.nonNavigableNearEnds(element) {
            value = end.qualifier().name().substring(6);
            ## integers between 0 and dimensions AND each value is unique
            if (end.qualifier().name().substring(1, 5) != "index" || (value < 0 || value >= numOfOrderValues)
                || (orderValues.index(value) != orderValues.rindex(value))) {
                error("CORBAArray", element.name(), "3.5.20", "The single non-navigable near AssociationEnd of a CORBAArray-stereotyped Class must have one or more qualifiers with the names \"index<i>\", where the <i> are contiguous integers starting from 0.");
            }
        }
        # [2]
        for end in self.umlOperations.navigableOppositeEnds(element) {
            if (end.multiplicity().range()[0].lower() != 1 || end.multiplicity().range()[0].upper() != 1) {
                error("CORBAArray", element.name(), "3.5.20", "The single navigable opposite AssociationEnd of a CORBAArray-stereotyped Class must have multiplicity 1..1.");
            }
        }
    }


    # =================================================================
    #   3.5.20 - Array - CORBAAnonymousArray
    # =================================================================
    proc checkCORBAAnonymousArray (self, element)
    {
        self.checkCORBAArray(element);
        # [1]
        numberEnds = 0;
        for end in self.umlOperations.navigableOppositeEnds(element) {
            if (numberEnds == 1 || end.multiplicity().range()[0].lower() != 1 || end.multiplicity().range()[0].upper() != 1) {
                error("CORBAAnonymousArray", element.name(), "3.5.20", "A CORBAAnonymousArray-stereotyped Class must have exactly one navigable opposite AssociationEnd whose multiplicity is 1..1.");
            }
            numberEnds = numberEnds + 1;
        }
    }


    # =================================================================
    #   3.5.21 - Fixed Type
    # =================================================================
    proc checkCORBAFixed (self, element)
    {
        self.checkCORBAStructuredType(element);
    }


    # =================================================================
    #   3.5.22 - Operation - oneway
    # =================================================================
    proc checkOperationOneway (self, operation)
    {
        # [1]
        numberReturn = 0;
        for param in operation.parameter() {
            if (param.kind() == UML.DataTypes.ParameterDirectionKind.pdk_out || param.kind() == UML.DataTypes.ParameterDirectionKind.pdk_inout) {
                error("oneway", operation.name(), "3.5.22", "A oneway-stereotyped Operation may have any out or inout Parameters.");
            }
            if (param.kind() == UML.DataTypes.ParameterDirectionKind.pdk_return) {
                if ((param.type().name() != "CORBA::void" && param.type().name() != "void") || numberReturn == 1) {
                    error("oneway", operation.name(), "3.5.22", "A oneway-stereotyped Operation must have a return Parameter of type CORBA::void");
                }
                numberReturn = numberReturn + 1;
            }
        }
        # [2]
        if (!self.umlOperations.isStereotyped(operation.owner(), "CORBAInterface")) {
            error("oneway", operation.name(), "3.5.22", "A oneway-stereotyped Operation must be owned by a CORBAInterface-stereotyped Class.");
        }
    }


    # =================================================================
    #   3.5.23 - Attribute - readonly
    # =================================================================
    proc checkAttributeReadonly (self, element)
    {
        for tag in element.tagged_value() {
            if (tag.type().name() == "setRaises") {
                error("readonly", element.name(), "3.5.23", "A readonly-stereotyped Attribute or a readonlyEnd-stereotyped AssociationEnd cannot raise a modify exception.");
            }
        }
    }


}
