# ====================================================================
#
# OpenCCM: The Open CORBA Component Model Platform
# Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
# Contact: openccm@objectweb.org
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Pierre Carpentier.
# Contributor(s): ______________________________________.
#
# ====================================================================
#
#  This class provides methods to check all the constraints
#  defined in the UML Profile for CORBA Components (Version 3.1).
#
# ====================================================================

import CORBA2Checker;
import ConstraintException;

class CORBA3Checker (CORBA2Checker.CORBA2Checker)
{

    # =================================================================
    # Empty constructor for the CORBA3Checker.
    # =================================================================
    proc __CORBA3Checker__(self)
    {
        self.__CORBA2Checker__();
    }


    # =================================================================
    #   Prints an error message.
    #   stereotype - The stereotype which doesn't respect the constraint
    #   cause - The name of the element which causes the exception
    #   section - The corresponding section of the specification
    #   message - The error message
    # =================================================================
    proc error(stereotype, cause, section, message) {
        throw ConstraintException.ConstraintException (stereotype, cause, "UML Profile for CORBA Components (Version 3.1)", section, message);
    }


    # =================================================================
    #   4.1.2.1 - Component
    # =================================================================
    proc checkCORBAComponent (self, element)
    {
        self.checkCORBAInterface(element);
        for feat in element.feature() {
            if (feat._is_a(UML.Core.BehavioralFeature)) {
                error("CORBAComponent", element.name(), "4.1.2.1", "A CORBAComponent cannot own operations.");
            }
        }
        correctInheritance = true;
        numberInheritComponent = 0;
        for g in element.generalization() {
            if (self.umlOperations.isStereotyped(g.parent(), "CORBAInterface")) {
                if (!self.umlOperations.isStereotyped(g, "CORBASupports")) {
                    error("CORBAComponent", element.name(), "4.1.2.1", "Each CORBAComponent inheritance from a CORBAInterface must be stereotyped CORBASupports");
                }
            } else if (self.umlOperations.isStereotyped(g.parent(), "CORBAComponent")) {
                numberInheritComponent = numberInheritComponent + 1 ;
            } else {
                correctInheritance = false;
            }
        }
        if (!correctInheritance || numberInheritComponent > 1) {
            error("CORBAComponent", element.name(), "4.1.2.1", "A CORBAComponent can only inherit from one CORBAComponent or CORBAInterface.");
        }
    }


    # =================================================================
    #   4.1.2.2 - Facet and Receptacle
    # =================================================================
    proc checkCORBAPort (self, element)
    {
        namePort = "noname";
        interfaceSide = "notset";
        componentSide = "notset";        
        for connect in element.connection() {
            if (self.umlOperations.isStereotyped(connect.participant(), "CORBAInterface")
                || self.umlOperations.isStereotyped(connect.participant(), "CORBAHome")) {
                interfaceSide = connect;
                namePort = connect.participant().name();
            }
            if (self.umlOperations.isStereotyped(connect.participant(), "CORBAComponent")) {
                componentSide = connect;
                namePort = connect.participant().name();
            }
        }
        try {
            namePort = element.name();
        } catch (Reflective.NotSet notSet) {
        }
        if (element.connection().length != 2) {
            error("CORBAPort", namePort, "4.1.2.2", "It's a binary association.");
        }
        if (interfaceSide == "notset" || componentSide == "notset") {
            error("CORBAPort", namePort, "4.1.2.2", "It's an association between a CORBAComponent and a CORBAInterface.");
        }
        if (componentSide.aggregation() != UML.DataTypes.AggregationKind.ak_composite) {
            error("CORBAPort", namePort, "4.1.2.2", "The CORBAComponent side is a composition.");
        }
        if (interfaceSide.name() == "" || interfaceSide.name().length == 0) {
            error("CORBAPort", namePort, "4.1.2.2", "The CORBAInterface role name must be defined.");
        }
    }


    # =================================================================
    #   4.1.2.2 - Facet and Receptacle - CORBAProvides
    # =================================================================
    proc checkCORBAProvides (self, element)
    {
        self.checkCORBAPort(element);
        for connect in element.connection() {
            if (self.umlOperations.isStereotyped(connect.participant(), "CORBAInterface")
                || self.umlOperations.isStereotyped(connect.participant(), "CORBAHome")) {
                if (connect.multiplicity().range()[0].lower() != 1 || connect.multiplicity().range()[0].upper() != 1) {
                    error("CORBAProvides", connect.participant().name(), "4.1.2.2", "The CORBAInterface side cardinality must be 1.");
                }
            }
        }
    }


    # =================================================================
    #   4.1.2.2 - Facet and Receptacle - CORBAUses
    # =================================================================
    proc checkCORBAUses (self, element)
    {
        self.checkCORBAPort(element);
        nameUses = "noname";
        for connect in element.connection() {
            if (self.umlOperations.isStereotyped(connect.participant(), "CORBAInterface")
                || self.umlOperations.isStereotyped(connect.participant(), "CORBAHome")) {
                lower = connect.multiplicity().range()[0].lower();
                upper = connect.multiplicity().range()[0].upper();
                multiple = "no";
                for tv in element.tagged_value() {
                    if (tv.type().tag_type() == "CORBAIsMultiple") {
                        multiple = tv.data_value();
                    }
                }
                try {
                    nameUses = element.name();
                } catch (Reflective.NotSet notSet) {
                    nameUses = connect.participant().name();
                }
                if (multiple != "yes" && multiple != "no") {
                    error("CORBAUses", nameUses, "4.1.2.2", "The CORBAIsMultiple tagged value has an incorrect value. \"Multiple\" value can be 'yes' or 'no'.");
                }
                if (multiple == "yes" && (lower != 1 || upper ==0 || upper == 1)) {
                    error("CORBAUses", nameUses, "4.1.2.2", "The CORBAInterface side cardinality must greater than one for multiple receptacles.");
                }
                if (multiple == "no" && (lower != 1 || upper != 1)) {
                    error("CORBAUses", nameUses, "4.1.2.2", "The CORBAInterface side cardinality must be 1 for simple receptacle.");
                }
            }
        }
    }


    # =================================================================
    #   4.1.2.3 - Event
    # =================================================================
    proc checkCORBAEvent (self, element)
    {
        self.checkCORBAValue(element);
    }


    # =================================================================
    #   4.1.2.3 - EventPort
    # =================================================================
    proc checkCORBAEventPort (self, element)
    {
        nameEventPort = "noname";
        eventSide = "notset";
        componentSide = "notset";
        for connect in element.connection() {
            if (self.umlOperations.isStereotyped(connect.participant(), "CORBAEvent")) {
                eventSide = connect;
                nameEventPort = connect.participant().name();
            }
            if (self.umlOperations.isStereotyped(connect.participant(), "CORBAComponent")) {
                componentSide = connect;
                nameEventPort = connect.participant().name();
            }
        }
        try {
            nameEventPort = element.name();
        } catch (Reflective.NotSet notSet) {
        }
        if (element.connection().length != 2) {
            error("CORBAEventPort", nameEventPort, "4.1.2.3", "It's a binary association.");
        }
        if (eventSide == "notset" || componentSide == "notset") {
            error("CORBAEventPort", nameEventPort, "4.1.2.3", "It's an association between a CORBAComponent and a CORBAEvent.");
        }
        if (componentSide.aggregation() != UML.DataTypes.AggregationKind.ak_composite) {
            error("CORBAEventPort", nameEventPort, "4.1.2.3", "The CORBAComponent side is a composition.");
        }
        if (eventSide.name() == "" || eventSide.name().length == 0) {
            error("CORBAEventPort", nameEventPort, "4.1.2.3", "The CORBAEvent side role name must be defined.");
        }
    }


    # =================================================================
    #   4.1.2.3 - CORBAConsumes
    # =================================================================
    proc checkCORBAConsumes (self, element)
    {
        self.checkCORBAEventPort(element);
        nameConsumes = "noname";
        lower = 0;
        upper = 0;
        connect = element.connection();
        if (self.umlOperations.isStereotyped(connect[0].participant(), "CORBAEvent")) {
            lower = connect[0].multiplicity().range()[0].lower();
            upper = connect[0].multiplicity().range()[0].upper();
            nameConsumes = connect[0].participant().name();
        } else {
            lower = connect[1].multiplicity().range()[0].lower();
            upper = connect[1].multiplicity().range()[0].upper();
            nameConsumes = connect[1].participant().name();
        }
        try {
            nameConsumes = element.name();
        } catch (Reflective.NotSet notSet) {
        }
        if (lower != 1 || upper != 1) {
            error("CORBAConsumes", nameConsumes, "4.1.2.3", "The CORBAEvent side cardinality must be 1.");
        }
    }


    # =================================================================
    #   4.1.2.3 - CORBAEmits
    # =================================================================
    proc checkCORBAEmits (self, element)
    {
        self.checkCORBAEventPort(element);
        nameEmits = "noname";
        lower = 0;
        upper = 0;
        connect = element.connection();
        if (self.umlOperations.isStereotyped(connect[0].participant(), "CORBAEvent")) {
            lower = connect[0].multiplicity().range()[0].lower();
            upper = connect[0].multiplicity().range()[0].upper();
            nameEmits = connect[0].participant().name();
        } else {
            lower = connect[1].multiplicity().range()[0].lower();
            upper = connect[1].multiplicity().range()[0].upper();
            nameEmits = connect[1].participant().name();
        }
        try {
            nameEmits = element.name();
        } catch (Reflective.NotSet notSet) {
        }
        if (lower != 1 || upper != 1) {
            error("CORBAEmits", nameEmits, "4.1.2.3", "The CORBAEvent side cardinality must be 1.");
        }
    }


    # =================================================================
    #   4.1.2.3 - CORBAPublishes
    # =================================================================
    proc checkCORBAPublishes (self, element)
    {
        self.checkCORBAEventPort(element);
        namePublishes = "noname";
        lower = 0;
        upper = 0;
        connect = element.connection();
        if (self.umlOperations.isStereotyped(connect[0].participant(), "CORBAEvent")) {
            upper = connect[0].multiplicity().range()[0].upper();
            namePublishes = connect[0].participant().name();
        } else {
            upper = connect[1].multiplicity().range()[0].upper();
            namePublishes = connect[1].participant().name();
        }
        try {
            namePublishes = element.name();
        } catch (Reflective.NotSet notSet) {
        }
        if (upper == 0) {
            error("CORBAPublishes", namePublishes, "4.1.2.3", "The CORBAEvent side cardinality can be 1 or more.");
        }
    }


    # =================================================================
    #   4.1.2.4 - Component Home
    # =================================================================
    proc checkCORBAHome (self, element)
    {
        self.checkCORBAInterface(element);
        numberCORBAManages = 0;
        for asso in self.umlOperations.associations(element) {
            if (self.umlOperations.isStereotyped(asso, "CORBAManages")) {
                numberCORBAManages = numberCORBAManages + 1;
                assoManages = asso;
                self.checkCORBAManages(assoManages);
            }
        }
        if (numberCORBAManages != 1) {
            error("CORBAHome", element.name(), "4.1.2.4", "Exactly one CORBAManages association for each Home.");
        }
        homeSideLower = 0;
        homeSideUpper = 0;
        compSideLower = 0;
        compSideUpper = 0;
        connect = assoManages.connection();
        if (self.umlOperations.isStereotyped(connect[0].participant(), "CORBAHome")
            && self.umlOperations.isStereotyped(connect[1].participant(), "CORBAComponent")) {
            homeSideLower = connect[0].multiplicity().range()[0].lower();
            homeSideUpper = connect[0].multiplicity().range()[0].upper();
            compSideLower = connect[1].multiplicity().range()[0].lower();
            compSideUpper = connect[1].multiplicity().range()[0].upper();
            managedComponent = connect[1].participant();
        } else if (self.umlOperations.isStereotyped(connect[1].participant(), "CORBAHome")
                   && self.umlOperations.isStereotyped(connect[0].participant(), "CORBAComponent")) {
            homeSideLower = connect[1].multiplicity().range()[0].lower();
            homeSideUpper = connect[1].multiplicity().range()[0].upper();
            compSideLower = connect[0].multiplicity().range()[0].lower();
            compSideUpper = connect[0].multiplicity().range()[0].upper();
            managedComponent = connect[0].participant();
        }
        if (homeSideLower != 1 || homeSideUpper != 1) {
            error("CORBAHome", element.name(), "4.1.2.4", "The CORBAHome side cardinality must be 1..1.");
        }
        if (compSideLower != 0 || compSideUpper == 0 || compSideUpper == 1) {
            error("CORBAHome", element.name(), "4.1.2.4", "The CORBAComponent side cardinality must be 0..n.");
        }
        correctInheritance = true;
        numberInheritHome = 0;
        for g in element.generalization() {
            if (self.umlOperations.isStereotyped(g.parent(), "CORBAHome")) {
                numberInheritHome = numberInheritHome + 1;
                superHome = g.parent();
                managedComponentBySuperHome = managedComponent(superHome);
                if (managedComponent != managedComponentBySuperHome
                    && !managedComponent._is_a(managedComponentBySuperHome._type)) {
                    error("CORBAHome", element.name(), "4.1.2.4", "If CORBAHome h1 inherits from CORBAHome h2 and h2 manages CORBAComponent C2 then h1 must manage C2 or any other component C1 that inherits from C2.");
                }
                primaryKey = "notset";
                if (assoManages._is_a(UML.Core.UmlAssociationClass))
                    primaryKey = assoManages;
                primaryKeySuperHome = primaryKey(superHome, managedComponentBySuperHome);
                if (primaryKeySuperHome != "notset" && primaryKey != primaryKeySuperHome && !primaryKey._is_a(primaryKeySuperHome._type)) {
                    error("CORBAHome", element.name(), "4.1.2.4", "If CORBAHome h1 inherits from h2, and CORBAHome h2 is associated with primary key k2 then h1 must be associated with k2 or with a primary key k1 that inherits from k2.");
                }
            } else if (self.umlOperations.isStereotyped(g.parent(), "CORBAInterface")) {
                if (self.umlOperations.isStereotyped(g, "CORBASupports")) {
                    error("CORBAHome", element.name(), "4.1.2.4", "Each CORBAHome inheritance from a CORBAInterface must be stereotyped CORBASupports.");
                }
            } else {
                correctInheritance = false;
            }
        }
        if (!correctInheritance || numberInheritHome > 1) {
            error("CORBAHome", element.name(), "4.1.2.4", "A CORBAHome can inherit from one CORBAHome at most.");
        }
    }


    # =================================================================
    #   Returns the component managed by a home
    # =================================================================
    proc managedComponent (home) {
        for asso in self.umlOperations.associationsOfClass(home) {
            if (self.umlOperations.isStereotyped(asso, "CORBAManages")) {
                connect = asso.connection();
                if (self.umlOperations.isStereotyped(connect[0].participant(), "CORBAHome")
                    && self.umlOperations.isStereotyped(connect[1].participant(), "CORBAComponent")) {
                    return connect[1].participant();
                } else if (self.umlOperations.isStereotyped(connect[1].participant(), "CORBAHome")
                    && self.umlOperations.isStereotyped(connect[0].participant(), "CORBAComponent")) {
                    return connect[0].participant();
                }
            }
        }
    }


    # =================================================================
    #   Returns the primary key association class of a home
    # =================================================================
    proc primaryKey (home, component) {
        all_association_class = UMLPackage.core_ref.association_class_ref.all_of_type_association_class;
        for assoCls in all_association_class {
            connect = assoCls.connection();
            if (connect.length == 2) {
                if ((connect[0] == home && connect[1] == component) || (connect[0] == component && connect[1] == home))
                    return assoCls;
            }
        }
        return "notset";
    }


    # =================================================================
    #   4.1.2.4 - Component Home - CORBAManages
    # =================================================================
    proc checkCORBAManages (self, element)
    {
        homeSide = "notset";
        componentSide = "notset";
        nameManages = "noname";
        for connect in element.connection() {
            if (self.umlOperations.isStereotyped(connect.participant(), "CORBAComponent")) {
                componentSide = connect;
                nameManages = connect.participant().name();
            }
            if (self.umlOperations.isStereotyped(connect.participant(), "CORBAHome")) {
                homeSide = connect;
                nameManages = connect.participant().name();
            }
            try {
                nameManages = element.name();
            } catch (Reflective.NotSet notSet) {
            }
        }
        if (homeSide == "notset" || componentSide == "notset") {
            error("CORBAManages", nameManages, "4.1.2.4", "It's an association between a CORBAHome and a CORBAComponent.");
        }
    }


    # =================================================================
    #   4.1.2.4 - Component Home - CORBAPrimaryKey
    # =================================================================
    proc checkCORBAPrimaryKey (self, element)
    {
        numberPublicAttribute = 0;
        for feat in element.feature() {
            if (feat._is_a(UML.Core.Attribute)) {
                if (feat.visibility() == UML.DataTypes.VisibilityKind.vk_public)
                    numberPublicAttribute = numberPublicAttribute + 1;
            }
        }
        if (numberPublicAttribute == 0)
            error("CORBAPrimaryKey", element.name(), "4.1.2.4", "A PrimaryKey must own a public attribute.");
    }


    # =================================================================
    #   4.1.2.4 - Component Home - CORBAHomeFactory
    # =================================================================
    proc checkCORBAHomeFactory (self, element)
    {
        for param in element.parameter() {
            if (param.kind() != UML.DataTypes.ParameterDirectionKind.pdk_in)
                error("CORBAHomeFactory", element.name(), "4.1.2.4", "A CORBAHomeFactory operation has only input parameters.");
        }
        if (!isStereotyped(element.owner(), "CORBAHome")) {
            error("CORBAHomeFactory", element.name(), "4.1.2.4", "A CORBAHomeFactory can only be defined in a CORBAHome.");
        }
    }


    # =================================================================
    #   4.1.2.4 - Component Home - CORBAHomeFinder
    # =================================================================
    proc checkCORBAHomeFinder (self, element)
    {
        for param in element.parameter() {
            if (param.kind() != UML.DataTypes.ParameterDirectionKind.pdk_in)
                error("CORBAHomeFinder", element.name(), "4.1.2.4", "A CORBAHomeFinder has only input parameters.");
        }
        if (!isStereotyped(element.owner(), "CORBAHome")) {
            error("CORBAHomeFinder", element.name(), "4.1.2.4", "A CORBAHomeFinder can only be defined in a CORBAHome.");
        }
    }


}
