/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

====================================================================*/

package org.objectweb.corba.uml.lib;

import org.objectweb.util.cmdline.lib.DefaultCommandLine;
import org.objectweb.corba.ast.api.FileScope;

/**
 * Implementation of the xmi2ir command.
 *
 * @author <a href="mailto:Pierre.Carpentier@lifl.fr">Pierre Carpentier</a>
 *
 * @version 1.0
 */

public class XMI2IR
  extends org.objectweb.corba.command.lib.CompilerBase
    implements org.objectweb.corba.uml.api.XMI2IR
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
 
    /** The Java IDL Script Interpreter. */
    fr.lifl.cim.jidlscript.JISInterpreter jis;

    /** The UML package factory. */
    org.omg.UML.UMLPackageFactory umlPackageFactory;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    XMI2IR()
    {
        // Calls the CompilerBase constructor.
        super(new DefaultCommandLine("xmi2ir",
                                     "xmi_uml_document",
                                     "Feed the OpenCCM Interface Repository with an XMI 1.1 UML 1.4 document",
                                     true),
              false);
        // Sets the preprocessor options.
        getPreprocessorApplication().getPreprocessor().addIncludeDirectory(System.getProperty("include.directory"));
        getPreprocessorApplication().getPreprocessor().addMacro(System.getProperty("dname.macro"));
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Initialization of the Java IDL Script Interpreter.
     *
     * @param orb The CORBA::ORB.
     */
    private void
    initJIS(org.omg.CORBA.ORB orb)
    {
        try {
            // Creation of the interpreter.
            jis = new fr.lifl.cim.jidlscript.JISInterpreter(orb);
            String csDirectory = System.getProperty("transformation.cs.dir");
            java.net.URL urlDirectory = Thread.currentThread().getContextClassLoader().getResource(csDirectory);
            fr.lifl.cim.jidlscript.lang.JISModuleLoader.loader.addURL(urlDirectory.toString());
        } catch (fr.lifl.cim.jidlscript.lang.JISError jisError) {
            getConsole().error("Error in Java IDL Script Interpreter initialization: " + jisError.getMessage());
        }
    }

    /**
     * Construction of the command to check the UML model.
     * @param iorUmlPackage The IOR of the UML Package.
     * @return The Java IDL Script command.
     */
    private String
    getCheckCommand(String iorUmlPackage)
    {
        StringBuffer checkCommand = new StringBuffer();
        checkCommand.append("import ModelChecker;");
        checkCommand.append("checker = ModelChecker.ModelChecker();");
        checkCommand.append("umlPackage = CORBA.Object(\"" + iorUmlPackage + "\");");
        checkCommand.append("checker.checkModel(umlPackage);");
        return checkCommand.toString();
    }

    /**
     * Construction of the command to create the UML model into the OpenCCM Interface Repository.
     *
     * @param iorUmlPackage The IOR of the UML Package.
     * @return The Java IDL Script command.
     */
    private String
    getCreateCommand(String iorUmlPackage)
    {
        StringBuffer createCommand = new StringBuffer();
        createCommand.append("import ASTFeeder;");
        createCommand.append("feeder = ASTFeeder.ASTFeeder();");
        createCommand.append("umlPackage = CORBA.Object(\"" + iorUmlPackage + "\");");
        createCommand.append("feeder.createModel(umlPackage);");
        return createCommand.toString();
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Application
    //
    // ==================================================================

    /**
     * Runs the application.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     */
    public int
    run(java.lang.String[] args)
    {
        String filename = args[0];
        // Inits the OpenCCM's Abstract Syntax Tree.
        if(!initAST())
            return -1;

        org.omg.CORBA.ORB orb = org.objectweb.ccm.CORBA.TheORB.getORB();
        initJIS(orb);
        initUMLRepository(orb);
        if (umlPackageFactory == null) {
            return -1;
        }
        
        try {
            // Inits the UML XMI Importer.
            org.omg.CORBA.Object model_package = umlPackageFactory.create_uml_package();
            org.omg.UML.xmi.UMLXMIImport xmiImport = new org.omg.UML.xmi.UMLXMIImport(true);

            // Reflective
            // xmiImport.initiate(model_package, orb);
            java.lang.reflect.Method meths [] = xmiImport.getClass().getMethods();
            for (int i=0 ; i<meths.length ; i++) {
                if (meths[i].getName().equals("initiate")) {
                    Object arguments [] = new Object[2];
                    arguments [0] = model_package;
                    arguments [1] = orb;
                    meths[i].invoke(xmiImport, arguments);
                }
            }
     
            // Parses the UML XMI file.
            getConsole().message("Reading from " + filename + "...");
            org.omg.CORBA.Object uml_package = (org.omg.CORBA.Object) xmiImport.parse(filename);

            String iorUmlPackage = orb.object_to_string(uml_package);

            // Checks if the model is correct.
            getConsole().message("Checking if the UML model is correct...");
            jis.eval(getCheckCommand(iorUmlPackage));
            getConsole().message("The UML model is correct.");

            // Creates the corresponding elements into the OpenCCM Interface Repository.
            getConsole().message("Feeding the UML model into the OpenCCM Interface Repository...");
            jis.eval(getCreateCommand(iorUmlPackage));
            getConsole().message("The UML model is fed into the OpenCCM Interface Repository.");

        } catch (fr.lifl.cim.jidlscript.lang.JISError e) {
            if (((fr.lifl.cim.jidlscript.lang.JISException)e.exception).getType().Name().equals("Throw")) {
                String message = ((fr.lifl.cim.jidlscript.lang.JISThrow)e.exception).eval().toString();
                if (message.equals("Reflective::NotSet()")) {
                    getConsole().error("[ERROR] There is a blank field in the UML model (e.g. name, visibility, multiplicity, etc.).");
                } else {
                    // ConstraintException or NotCreatedException
                    java.util.StringTokenizer token = new java.util.StringTokenizer(message, "\n");
                    while (token.hasMoreTokens()) {
                        getConsole().error("[ERROR] " + token.nextToken());
                    }
                }
            } else {
                getConsole().error("[ERROR] An error occurs during the importation of the UML model. Check if your XMI file is correct (" + e.exception + ").");
            }
            return -1;
        } catch (org.omg.mof.Reflective.MofError mofError) {
            getConsole().error("[ERROR] An error occurs during the starting of the UML repository. Check your environment.");
            return -1;
        } catch (Exception exception) {
            getConsole().error("[ERROR] An error occurs during the reading of the UML model (" + exception.getMessage() + ").");
            return -1;
        }

        // All is OK.
        return 0;
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.CommandOnIR3
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.CommandOnAST
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Compiler
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.corba.command.api.XMI2IR
    //
    // ==================================================================

    /**
     * Starts the UML repository server.
     *
     * @param orb The CORBA::ORB.
     */
    public void
    initUMLRepository(org.omg.CORBA.ORB orb)
    {
        // Feeds the IR3 with the UML model if it is not present.
        if (getComponentRepository().lookup("UML") == null) {
            compile(System.getProperty("repository.uml.idl"));
        } else {
            getConsole().message("The UML.idl file is already loaded into the OpenCCM Interface Repository.");
        }

        try {
            // Get reference to rootpoa & activate the POAManager
            org.omg.PortableServer.POA rootpoa = org.omg.PortableServer.POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
            rootpoa.the_POAManager().activate();
            
            // Create servant and register it with the ORB
            org.omg.UML.UMLPackageFactoryImpl _model_package_factory = new org.omg.UML.UMLPackageFactoryImpl();
            
            // Get object reference from the servant
            org.omg.CORBA.Object ref = rootpoa.servant_to_reference(_model_package_factory);
            umlPackageFactory = org.omg.UML.UMLPackageFactoryHelper.narrow(ref);
        } catch (java.lang.Exception exception) {
            getConsole().error("Error in UML Package Factory initialization: " + exception.getMessage());
        }
    }

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     */
    public static void
    main(String[] args)
    {
        XMI2IR xmi2ir = new XMI2IR();
        xmi2ir.runMain(args);
    }

}
