/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.naming.wrappers.common;

/** To use the Context interface. */
import org.objectweb.util.browser.naming.api.Context;

/** To use the Wrapper interface. */
import org.objectweb.util.browser.naming.api.Wrapper;

/** To use the Name interface. */
import org.objectweb.util.browser.naming.api.Name;

/** To use the DefaultName class. */
import org.objectweb.util.browser.naming.wrappers.common.DefaultName;

/** To use the Entry interface. */
import org.objectweb.util.browser.naming.api.Entry;

/** To use the DefaultEntry class. */
import org.objectweb.util.browser.naming.wrappers.common.DefaultEntry;

/** To use the EntryIterator interface. */
import org.objectweb.util.browser.naming.api.EntryIterator;

/** To use the Decoder API */
import org.objectweb.util.browser.naming.api.Decoder;

/**
 * Basic implementation of the Context interface
 * for intercepting message
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class ContextDecoderManager 
    implements Context, Wrapper {
	
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The decoder. */
    protected Decoder decoder_;

    /** The delegate */
    protected Context delegate_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Creates an Entry.
     *
     * @param value The value of the entry to create.
     * @param id The id of the name of the entry to create.
     *
     * @return A new Entry.
     *
     * @postcondition
     *              return.getOWValue() == value
     * and return.getOWName().getOWID() == id
     * and        return.getOWContext() == this;
     */
    protected Entry createEntry(Object value, String id) {
        return new DefaultEntry(value, new DefaultName(id), this);
    }

    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

    /**
     * Gets the scope of the target context.
     *
     * This method does not update the scope of the target context.
     *
     * @return The scope of the target context, or null if the
     *         target context has no scope.
     *
     * @postcondition return == getOWScope();
     */
    public Context getOWScope() {
        // TODO
        return delegate_.getOWScope();
    }

    /**
     * Gets a local entry in the target context identified by <code>name</code>,
     * or null if no entry exists under name <code>name</code>.
     *
     * Here is a local resolution in the target context.
     *
     * This method does not modify the target context.
     *
     * @param name The name of the entry to be retrieved.
     *
     * @return The corresponding entry, if any; or null otherwise.
     *
     * @postcondition
     *      result.getOWName().getOWId().equals(name)
     *  and result.getOWContext() == this
     *  and this.isNotModified();
     */
    public Entry getOWLocalEntry(String name) {
        Entry e = delegate_.getOWLocalEntry(name);
        return (e != null) ? createEntry(decoder_.decode(e.getOWValue()),e.getOWName().getOWId()) : null;
    }

    /**
     * Gets the value of a local entry in the target context identified by
     * <code>name</code>, or null if no entry exists under name
     * <code>name</code>.
     *
     * This method is equivalent to
     * {@link #getOWLocalEntry(String) <code>getOWLocalEntry(name)</code>}
     * {@link Entry#getOWValue() <code>.getOWValue()</code>}.
     *
     * This method does not modify the target context.
     *
     * @param name The name of the entry to be retrieved. 
     * The form of the informations are "id:kind"
     *
     * @return The corresponding entry value, if any; or null otherwise.
     *
     * @postcondition
     *     ( getOWLocalEntry(name) == null && result == null )
     *  or result == getOWLocalEntry(name).getOWValue());
     */
    public Object getOWLocalValue(String name) {
        Object object = delegate_.getOWLocalValue(name);
        return (object != null) ? decoder_.decode(object) : null;
    }

    /**
     * Gets the entry in the target context identified by <code>name</code>,
     * or null if no entry exists under name <code>name</code>.
     *
     * This method does not modify the given name and the target context.
     *
     * @param name The name of the entry to be retrieved.
     *
     * @return The corresponding entry, if any; or null otherwise.
     *
     * @postcondition
     *      result.getOWName().equals(name)
     *  and name.isNotModified()
     *  and this.isNotModified();
     */
    public Entry getOWEntry(Name name) {
        // TODO
        return delegate_.getOWEntry(name);
    }

    /**
     * Gets the value of an entry in the target context identified by
     * <code>name</code>, or null if no entry exists under name
     * <code>name</code>.
     *
     * This method is equivalent to
     * {@link #getOWEntry(Name) <code>getOWEntry(name)</code>}
     * {@link Entry#getOWValue() <code>.getOWValue()</code>}.
     *
     * This method does not modify the given name and the target context.
     *
     * @param name The name of the entry to be retrieved.
     *
     * @return The corresponding entry, if any; or null otherwise.
     *
     * @postcondition
     *     ( getOWEntry(name) == null && result == null )
     *  or result == getOWEntry(name).getOWValue());
     */
    public Object getOWValue(Name name) {
        // TODO
        return delegate_.getOWValue(name);
    }

    /**
     * Returns a new {@link EntryIterator EntryIterator} to
     * iterate on the entries contained by the target context.
     *
     * @return A new EntryIterator.
     */
    public EntryIterator getOWEntries() {
        EntryIteratorDecoderManager entryIteratorDecoder =
            new EntryIteratorDecoderManager(this);
        entryIteratorDecoder.setDecoder(decoder_);
        entryIteratorDecoder.setDelegate(delegate_.getOWEntries());
        return entryIteratorDecoder;
    }

    // ==================================================================
    //
    // Additional public methods for Wrapper interface.
    //
    // ==================================================================

    /**
     * Fix the wrapped object
     *
     * @param object The object to wrap
     *
     */
    public void setWrapped(Object object) {
        //((Wrapper)delegate_).setWrapped(decoder_.decode(object));
         ((Wrapper) delegate_).setWrapped(object);
    }

    /**
     * Gets the wrapped object
     *
     * @return The wrapped object
     *
     */
    public Object getWrapped() {
        return ((Wrapper) delegate_).getWrapped();
    }

    /**
     * Gets the hashCode of the wrapped object
     *
     * @return The hashCode of the wrapped object
     * 
     */
    public int getWrappedHashCode() {
        return ((Wrapper) delegate_).getWrappedHashCode();
    }

    // ==================================================================
    //
    // Additional public methods.
    //
    // ==================================================================

    /**
     * Fixes the delegate of the decoder
     */
    public void setDelegate(Context context) {
        delegate_ = context;
    }

    /**
     * Fixes the decoder
     */
    public void setDecoder(Decoder decoder) {
        decoder_ = decoder;
    }
}
