/*====================================================================

ObjectWeb Util Preprocessor Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cpp.lib;

// Package dependencies.
import java.util.Map;
import java.util.Hashtable;
import java.util.Vector;
import java.io.File;
import java.io.OutputStream;

import org.objectweb.util.cpp.api.Preprocessor;
import org.objectweb.util.cmdline.lib.DefaultConsoleHolder;
import org.objectweb.util.misc.api.ExceptionWrapper;


/**
 * Abstract base class for Preprocessor instances.
 *
 * This provides the following properties:
 *
 * - GenerateLineInformation: Indicates if #line information should be generated.
 *
 * - IncludeDirectory: The list of directories where included files
 *                     are searched.
 *
 * - Macro: The list of defined macros.
 *
 * This provides two preprocess operations.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

abstract public class PreprocessorBase
              extends DefaultConsoleHolder
           implements org.objectweb.util.cpp.api.Preprocessor
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The GenerateLineInformation property. */
    protected boolean generateLineInformation_;

    /** Arguments for the preprocessor. */
    protected Vector arguments_;

    /** The list of include directories. */
    protected Vector includeDirectories_ ;

    /** The list of declared macros. */
    protected Hashtable macros_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    PreprocessorBase()
    {
        // Inits internal state.
        generateLineInformation_ = false;
        arguments_ = new Vector();
        includeDirectories_ = new Vector();
        macros_ = new Hashtable();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cpp.api.PreprocessorOperations
    //
    // ==================================================================

    /**
     * Preprocesses a file.
     *
     * @param fileName The name of the file to preprocess.
     *
     * @return The file containing the output file.
     */
    public java.io.File
    preprocess(String fileName)
    {
        File tmpFile = null;
        try
        {
            // Creates a temp file.
            tmpFile = File.createTempFile("OW_preprocessor",".tmp");
        }
        // Wraps any java.io.IOException exception.
        catch(java.io.IOException exc)
        {
            throw new ExceptionWrapper(exc);
        }

        boolean status = preprocess(fileName, tmpFile);

	// If error then deletes the tmp file.
        if (!status)
        {
            tmpFile.delete();
            return null;
        }

        // Returns the tmp file.
	return tmpFile;
    }

    /**
     * Preprocesses a file.
     *
     * @param fileName The name of the file to preprocess.
     * @param tmpFile The file containing the output file.
     *
     * @return True if OK.
     */
    abstract public boolean
    preprocess(String fileName,
               java.io.File tmpFile);

    /**
     * Preprocesses a file.
     *
     * Must be defined in subclasses.
     *
     * @param fileName The name of the file to preprocess.
     * @param output The output stream.
     */
    abstract public boolean
    preprocess(String fileName,
               java.io.OutputStream output);

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cpp.api.Preprocessor
    //
    // ==================================================================

    /**
     * Obtains the GenerateLineInformation property.
     *
     * @return The GenerateLineInformation property.
     */
    public boolean
    getGenerateLineInformation()
    {
        return generateLineInformation_;
    }

    /**
     * Sets the GenerateLineInformation property.
     *
     * @param b The GenerateLineInformation property.
     */
    public void
    setGenerateLineInformation(boolean b)
    {
        generateLineInformation_ = b;
    }

    /**
     * Adds a directory to the list of include directories.
     *
     * @param directoru The directory to add.
     */
    public void
    addIncludeDirectory(String directory)
    {
        arguments_.addElement("-I" + directory);
        includeDirectories_.addElement(directory);
    }

    /**
     * Obtains the list of include directories.
     *
     * @return The list of include directories.
     */
    public String[]
    getIncludeDirectory()
    {
        return (String[])includeDirectories_.toArray(new String[0]);
    }

    /**
     * Sets the list of include directories.
     *
     * @param directories The list of include directories.
     */
    public void
    setIncludeDirectory(String[] directories)
    {
        // Reinits the list of include directories.
        includeDirectories_ = new Vector();

        // Adds each directory.
	for(int i=0; i<directories.length; i++)
        {
            arguments_.addElement("-I" + directories[i]);
            addIncludeDirectory(directories[i]);
        }
    }

    /**
     * Adds a macro (name - 1) to the list of declared macros.
     *
     * @param name The name of the macro to add.
     */
    public void
    addMacro(String name)
    {
        arguments_.addElement("-D" + name);
        macros_.put(name, "1");
    }

    /**
     * Adds a macro (name - value) to the list of declared macros.
     *
     * @param name The name of the macro to add.
     * @param value The value of the macro to add.
     */
    public void
    addMacro(String name,
             String value)
    {
        arguments_.addElement("-D" + name + '=' + value);
        macros_.put(name, value);
    }

    /**
     * Removes a macro (name - value) from the list of declared macros.
     *
     * @param name The name of the macro to remove.
     */
    public void
    removeMacro(String name)
    {
        arguments_.addElement("-U" + name);
        macros_.remove(name);
    }

    /**
     * Obtains the list of declared macros.
     *
     * @return The list of declared macros.
     */
    public java.util.Map
    getMacro()
    {
        // TODO
        throw new RuntimeException("NOT IMPLEMENTED!!!");
    }

    /**
     * Sets the list of declared macros.
     *
     * @param macros The list of declared macros.
     */
    public void
    setMacro(java.util.Map macros)
    {
        // TODO
        throw new RuntimeException("NOT IMPLEMENTED!!!");
    }
}
