/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

====================================================================*/

package org.objectweb.util.launcher ;


import java.lang.reflect.Method ;

import java.util.Properties ;



/**
 * Implementation of a particular Java Command which takes into account
 * <ul>
 *   <li>an Identifier for the command</li>
 *   <li>a bootstrap class name for executing the command</li>
 *   <li>a specific classloader where class are stored</li>
 *   <li>a list of java properties to associate to the command</li>
 *   <li>a list of arguments to give to the bootstrap class</li>
 *   <li>a particular mode of execution</li>
 * </ul>
 *
 * <p>
 * This command represent most of the command which could be executed 
 * in the context of Java developements
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.2
 */
public class CommandJava
{
    /** Identifier of the command */
    protected String name_ ;
    /** name of the bootstrap class */
    protected String class_ ;
    /** classloader used */
    protected XBootClassLoader loader_ ;
    /** set of java properties */     
    protected Properties properties_ ;
    /** set of arguments for the bootstrap method */
    protected StringList args_ ;
    /** mode of execution */
    protected Mode mode_;

    /**
     * Default constructor
     */
    public CommandJava() {
        this(null);
    }

    public CommandJava(XBootClassLoader loader) {
        name_ = "default" ;
        class_ = "";
        args_ = new StringList();        
        properties_ = new Properties();
        mode_ = new ModeThread();
        loader_ = loader;
    }

    /**
     * Defines the identifier of the command
     *
     * @param value - identifier of the command ("default" by default)
     */
    public void setName(String value) { this.name_ = value ; }

    /**
     * Provides the identifier of the command
     * 
     * @return the identifier of the command ("default" by default)
     */
    public String getName() {return this.name_ ; }

    /**
     * Defines the Java class to execute
     *
     * @param value - full name of the java class
     */
    public void setClassname(String value) { 
        TraceSystem.get("launcher").debug(getName()+": Setting classname to "+value);
        this.class_ = value ; 
    }

    /**
     * Retrieves the name of the Java class to run
     *
     * @return full name of the java class
     */
    public String getClassname() { return this.class_ ; }


    /**
     * Defines the classloader to use when executing the application
     *
     * @param value - the classloader
     */
    public void setLoader(XBootClassLoader value) { this.loader_ = value ; }

    /**
     * Retrieves the classloader used when executing the application
     *
     * @return the classloader used
     */
    public XBootClassLoader getLoader() { return this.loader_ ; }


    /**
     * Defines the Java properties to load in the Java Virtual Machine
     *
     * @param value - the properties to set
     */
    public void addProperties(Properties value) { 
        TraceSystem.get("launcher").debug(getName()+": Adding properties "+value);
        this.properties_.putAll(value) ; 
    }

    /**
     * Retrieves the properties defined for this application
     *
     * @return the properties defined
     */
    public Properties getProperties() { return this.properties_ ; }


    /**
     * Defines the command line options for the main method of the application
     *
     * @param value - the options to specify
     */
    public void addArguments(StringList value) { 
        TraceSystem.get("launcher").debug(getName()+": Adding arguments "+value);
        this.args_.addAll(value) ; 
    }

    /**
     * Retrieves the command line options for the main method of the application
     *
     * @return the options specified
     */
    public StringList getArguments() { return this.args_ ; }


    /**
     * Defines the mode of execution of the command
     *
     * @param value - the mode of execution to use (ModeNormal by default)
     */
    public void setMode(Mode value) { 
        TraceSystem.get("launcher").debug(getName()+": Setting mode to "+value);
        this.mode_ = value ; }

    /**
     * Provides the mode of execution of the command
     *
     * @return the mode of execution used (ModeNormal by default)
     */
    public Mode getMode() { return this.mode_ ; }



    /** the initial ClassLoader used */
    protected ClassLoader systemLoader_ ;

    /**
     * Provides the type "String[]"
     *
     * @return an instance of the String[] type 
     */
    private Class getStringArrayType() { return (Class)(new String[0]).getClass();}

    /**
     * Run the application (must not be called)
     *
     * @throws LauncherException is raised when the Launcher wasn't able to launch 
     * the application
     */
    protected void execute()
        throws LauncherException
    { // Retrieve the public static void main(String[]) method of the class
        Class class_instance = null;
        TraceSystem.get("launcher").info(getName()+": Launching the "+getClassname()+" class");
//         TraceSystem.get("launcher").debug("ClassLoader used: "+getLoader());
        try {
            class_instance = Class.forName(getClassname(), false, getLoader()) ;
        } catch (java.lang.ClassNotFoundException ex) {
            throw new LauncherException(ex);
        }
        Method method = null;
        try {            
            Class[] params = {getStringArrayType()} ;
            method = class_instance.getMethod("main", params);
        } catch (java.lang.NoSuchMethodException ex) {
            throw new LauncherException(ex);
        }

        // Sets loader_ as Current Class Loader for the application
        this.systemLoader_ = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(getLoader());
        System.getProperties().putAll(getProperties());

        TraceSystem.get("launcher").debug("Options used: "+getArguments());
        try { // invoke the method main
            Object[] params = {getArguments().toArray()} ;
            method.invoke(null, params);
        } catch (java.lang.IllegalAccessException ex) {
            throw new LauncherException(ex);
        } catch (java.lang.reflect.InvocationTargetException ex) {
            throw new LauncherException(ex);
        }
    }


    /**
     * Run the application
     */
    public void run() {
        this.mode_.execute(this);
    }

    /**
     * Rollback the ClassLoader
     */
    protected void finalize() {
//         Thread.currentThread().setContextClassLoader(this.systemloader_);
    }


    /**
     * Returns true if the two command are using the same parameters
     *
     * @param obj - the CommandJava to compare
     */
    public boolean equals(Object obj) {
        if (obj instanceof CommandJava) {
            CommandJava cmd = (CommandJava) obj ;
            return ((getName().equals(cmd.getName()))
                    && (getClassname().equals(cmd.getClassname()))
                    && (getLoader().equals(cmd.getLoader()))
                    && (getProperties().equals(cmd.getProperties()))
                    && (getArguments().equals(cmd.getArguments())));
        }
        return false ;
    }
}
