/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

====================================================================*/

package org.objectweb.util.launcher.parser ;


import java.util.StringTokenizer;
import java.util.List;
import java.util.Iterator;
import java.util.HashMap;
import java.util.StringTokenizer;

import java.net.URL ;

import org.objectweb.util.launcher.CommandBuilder ;
import org.objectweb.util.launcher.CommandJava ;
import org.objectweb.util.launcher.LauncherException ;
import org.objectweb.util.launcher.TraceSystem ;


/**
 * This class is an implementation of the Parser interface for providing 
 * an XML parser using Zeus library.<BR>
 * <p>
 * This library uses the Zeus library for parsing the file
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.1
 */
public class XMLParserImpl
    implements Parser
{
    /** java properties are identified by separator value in XML files */
    public static final String separator = "@" ;


    /** List of parsed file (with includes) */
    protected java.util.ArrayList parsed_files_ ;
    /** List of elements composing the xml file */
    protected HashMap elements_;

    /**
     * Default constructor
     */
    public XMLParserImpl() {
        parsed_files_ = new java.util.ArrayList();
        elements_ = new HashMap();
    }


    /**
     * Default constructor
     */
    public XMLParserImpl(String url) {
        this();
        load(url);
    }


    /**
     * Reads a file using the Zeus library
     *
     * @param url - the url of the file to parse
     */
    protected Launcher readFile(String url) { 
        try {
            return readFile(new java.net.URL(url)); 
        } catch (Exception ex) {
            TraceSystem.get("decoder").error("Exception raised: "+ex);
            throw new LauncherException(ex);
        }
    }

    /**
     * Reads a file using the Zeus library
     *
     * @param url - the url of the file to parse
     */
    protected Launcher readFile(URL url) {
        TraceSystem.get("parser").info("Reading File: "+url);
        try {            
            return LauncherUnmarshaller.unmarshal(url.openStream()) ;
        } catch (java.io.IOException e) {
            TraceSystem.get("decoder").error("[I/O Error] : "+e);
            throw new LauncherException(e);
        }
    }

    /**
     * Parses an XML file and load the content of the file in
     *
     * @param config - the parsed file
     */
    protected void parseFile(Launcher config) {
        TraceSystem.get("parser").info("Parsing Zeus Configuration: "+config);
        if (config.getIncludeList()!=null) {
            for(Iterator i = config.getIncludeList().iterator() ; i.hasNext() ;) {
                Include incl = (Include)i.next();            
                if ( !parsed_files_.contains(incl.getUrl()) ) {
                    parsed_files_.add(incl.getUrl());
                    parseFile(readFile(incl.getUrl()));
                }
            }
            TraceSystem.get("parser").debug("Include list loaded: "+elements_.keySet());
        }

        if (config.getClasspathList()!=null) {
            for(Iterator i = config.getClasspathList().iterator() ; i.hasNext() ; ) {
                Classpath c = (Classpath)i.next();
                elements_.put(c.getId(), c);
            }
            TraceSystem.get("parser").debug("Classpath list loaded: "+elements_.keySet());
        }

        if (config.getArgumentsList()!=null) {
            for(Iterator i = config.getArgumentsList().iterator() ; i.hasNext() ; ) {
                Arguments a = (Arguments)i.next();
                elements_.put(a.getId(), a);
            }       
            TraceSystem.get("parser").debug("Arguments list loaded: "+elements_.keySet());
        }

        if (config.getPropertiesList()!=null) {
            for(Iterator i = config.getPropertiesList().iterator() ; i.hasNext() ; ) {
                Properties p = (Properties)i.next();
                elements_.put(p.getId(), p);
            }
            TraceSystem.get("parser").debug("Properties list loaded: "+elements_.keySet());
        }

        if (config.getRunList()!=null) {
            for(Iterator i = config.getRunList().iterator() ; i.hasNext() ; ) {
                Run r = (Run)i.next();
                elements_.put(r.getId(), r);
            }
            TraceSystem.get("parser").debug("Run list loaded: "+elements_.keySet());
        }

        if (config.getOptionList()!=null) {
            for(Iterator i = config.getOptionList().iterator() ; i.hasNext() ; ) {
                Option r = (Option)i.next();
                elements_.put(r.getId(), r);
            }
            TraceSystem.get("parser").debug("Option list loaded: "+elements_.keySet());
        }
    }


    /**
     * Replace tags @NAME@ in a string.<BR>
     * NAME must be a jave property.
     *
     * @param tagged - The string to replace.
     *
     * @return A new String with all tag replacements.
     */
    protected String replaceTag(String tagged) {
        StringTokenizer toke = new StringTokenizer(tagged,separator,false);
        StringBuffer result = new StringBuffer(toke.hasMoreTokens()?toke.nextToken():tagged);
        while(toke.hasMoreTokens()) {
            result.append(System.getProperty(toke.nextToken()));
            if (toke.hasMoreTokens())
                result.append(toke.nextToken());
        }
        return result.toString();
    }

    /**
     * Loads the content of the classpath into the description
     *
     * @param classpath - the classpath to load
     * @param description - the description where classpath should be loaded
     */
    protected void loadClasspath(Classpath classpath, OptionDescription description) {
        for (Iterator i = classpath.getPathList().iterator() ; i.hasNext() ; ) {
            Path path = (Path)i.next();
            if (path.getUrl() != null)
                description.addClass(path.getUrl());
            else
                loadClasspath((Classpath)elements_.get(path.getClasspath()),
                              description);  
        }
    }

    /**
     * Loads the content of the arguments into the description
     *
     * @param args - the arguments to load
     * @param description - the description where arguments should be loaded
     */
    protected void loadArguments(Arguments args, OptionDescription description) {
        for(Iterator i = args.getArgumentList().iterator() ; i.hasNext() ; ) {
            Argument arg = (Argument)i.next();
            if (arg.getValue() != null)
                description.addArgument(replaceTag(arg.getValue()));
            else
                loadArguments((Arguments)elements_.get(arg.getArguments()), 
                              description );
        }
    }

    /**
     * Loads the content of the properties into the description
     *
     * @param props - the properties to load
     * @param description - the description where properties should be loaded
     */
    protected void loadProperties(Properties props, OptionDescription description) {
        for(Iterator i = props.getPropertyList().iterator() ; i.hasNext() ; ) {
            Property prop = (Property)i.next();
            if ((prop.getName() != null) && (prop.getValue() != null))
                description.addProperty(prop.getName(), 
                                        replaceTag(prop.getValue()));
            else
                loadProperties((Properties)elements_.get(prop.getProperties()), 
                               description );
        }
    }


    /**
     * Loads an XML file 
     *
     * @param url - the url of the file to load
     */
    public void load(String url) {
        parseFile(readFile(url));
    }

    public OptionDescription getOption(String id){
        TraceSystem.get("parser").info("Retrieving option "+id);
        OptionDescription description = new OptionDescription();
        Option option = (Option) elements_.get(id);
        if (option==null) throw new LauncherException(new OptionNotFoundException("Option "+id+" not found"));
        
        if (option.getClasspath()!=null) {
            for (StringTokenizer st = new StringTokenizer(option.getClasspath());st.hasMoreTokens();){
                loadClasspath((Classpath)elements_.get(st.nextToken()), (OptionDescription)description);
            }
            TraceSystem.get("parser").debug("Loading classpath: "+description.getClasses());
        }
        if (option.getArguments()!=null) {
            for (StringTokenizer st = new StringTokenizer(option.getArguments());st.hasMoreTokens();){
                loadArguments((Arguments)elements_.get(st.nextToken()), (OptionDescription)description);
            }
            TraceSystem.get("parser").debug("Loading arguments: "+description.getArguments());
        }
        if (option.getProperties()!=null) {
            for (StringTokenizer st = new StringTokenizer(option.getProperties());st.hasMoreTokens();){
                loadProperties((Properties)elements_.get(st.nextToken()), (OptionDescription)description);
            }
            TraceSystem.get("parser").debug("Loading properties: "+description.getProperties());
        }
        return description;
    }

    /**
     * Provides the description of the run identified by the value of id
     *
     * @param id - the identifier of the run
     *
     * @return the description of the run
     */
    public RunDescription getRun(String id) {
        TraceSystem.get("parser").info("Retrieving run "+id);
        RunDescription description = new RunDescription();
        Run run = (Run) elements_.get(id);
        if (run==null) throw new LauncherException(new RunNotFoundException("Run "+id+" not found"));
        
        description.setMainclass(run.getMainclassname());
        TraceSystem.get("parser").debug("Loading Mainclass: "+run.getMainclassname());
        if (run.getClasspath()!=null) {
            for (StringTokenizer st = new StringTokenizer(run.getClasspath());st.hasMoreTokens();){
                loadClasspath((Classpath)elements_.get(st.nextToken()), (OptionDescription)description);
            }
            TraceSystem.get("parser").debug("Loading classpath: "+description.getClasses());
        }
        if (run.getArguments()!=null) {
            for (StringTokenizer st = new StringTokenizer(run.getArguments());st.hasMoreTokens();){
                loadArguments((Arguments)elements_.get(st.nextToken()), (OptionDescription)description);
            }
            TraceSystem.get("parser").debug("Loading arguments: "+description.getArguments());
        }
        if (run.getProperties()!=null) {
            for (StringTokenizer st = new StringTokenizer(run.getProperties());st.hasMoreTokens();){
                loadProperties((Properties)elements_.get(st.nextToken()), (OptionDescription)description);
            }
            TraceSystem.get("parser").debug("Loading properties: "+description.getProperties());
        }
        return description;
    }
}
